@@ -28,9 +28,13 @@ G_BEGIN_DECLS
 
 typedef void (*JTFActivateFunc)(GtkTreePath *activated_path, gpointer user_data);
 
-void xfmedia_jump_to_file_show(GtkWindow *parent, GtkTreeModel *model,
-        gint column, JTFActivateFunc activate_func, gpointer callback_data);
-
+GtkWidget *xfmedia_jump_to_file_get_widget(GtkTreeView *treeview,
+        GtkTreeModel *model, gint column, JTFActivateFunc activate_func,
+        gpointer callback_data);
+        
+gboolean xfmedia_jump_to_file_filter_visible_func(GtkTreeModel *model,
+        GtkTreeIter *itr, gpointer data);
+        
 G_END_DECLS
 
 #endif
