@@ -66,7 +66,7 @@
 #define EXO_API_SUBJECT_TO_CHANGE
 #include <exo/exo.h>
 
-#include "gtkxine.h"
+#include "xfmedia-xine.h"
 
 #include "main.h"
 #include "mainwin.h"
@@ -74,6 +74,7 @@
 #include <xfmedia/xfmedia-settings.h>
 #include "xfmedia-internal.h"
 #include "remote.h"
+#include <xfmedia/xfmedia-remote-client.h>
 #include "mediamarks.h"
 #include "playlist-files.h"
 /*#include "equaliser.h"*/
@@ -81,13 +82,13 @@
 static void mainwin_destroy_cb(GtkWidget *w, gpointer user_data);
 static void vwin_destroy_cb(GtkWidget *w, gpointer user_data);
 
-static XfMediaMainwin *g_mwin = NULL;
+static XfmediaMainwin *g_mwin = NULL;
 static pid_t main_pid = 0;
 static gboolean quitting = FALSE;
 static guint mwin_dest_id = 0, vwin_dest_id = 0;
 
 void
-xfmedia_quit(XfMediaMainwin *mwin, XfMediaQuitMode quit_mode)
+xfmedia_quit(XfmediaMainwin *mwin, XfmediaQuitMode quit_mode)
 {
     GtkWidget *main_window, *video_window;
     
@@ -140,7 +141,7 @@ xfmedia_quit(XfMediaMainwin *mwin, XfMediaQuitMode quit_mode)
 static void
 mainwin_destroy_cb(GtkWidget *w, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     
     DBG("entering");
     
@@ -150,7 +151,7 @@ mainwin_destroy_cb(GtkWidget *w, gpointer user_data)
 static void
 vwin_destroy_cb(GtkWidget *w, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     
     DBG("entering");
     
@@ -172,7 +173,7 @@ sighandler_cb(int sig)
 static void
 session_save_yourself_cb(ExoXsessionClient *client, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     
     xfmedia_quit(mwin, XFMEDIA_QUIT_MODE_EXT);
 }
@@ -190,6 +191,8 @@ do_startup_notify(GtkWidget *window)
     ctx = sn_launchee_context_new_from_environment(sdpy,
             gdk_screen_get_number(gtk_widget_get_screen(window)));
     if(ctx) {
+        if(!GTK_WIDGET_REALIZED(window))
+            gtk_widget_realize(window);
         sn_launchee_context_setup_window(ctx, GDK_WINDOW_XWINDOW(window->window));
         sn_launchee_context_complete(ctx);
         sn_launchee_context_unref(ctx);
@@ -201,7 +204,7 @@ do_startup_notify(GtkWidget *window)
 static gboolean
 initial_autoplay(gpointer data)
 {
-    XfMediaMainwin *mwin = data;
+    XfmediaMainwin *mwin = data;
     
     gdk_threads_enter();
     xfmedia_mainwin_play_file_at_index(mwin, 0);
@@ -211,7 +214,7 @@ initial_autoplay(gpointer data)
 }
 
 static void
-print_usage(const gchar *argv0)
+xfmedia_print_usage(const gchar *argv0)
 {
 	gchar *cmd;
 	
@@ -227,16 +230,15 @@ print_usage(const gchar *argv0)
 	g_print(_("    -v, --verbose        Instruct xine to print useful status information.\n"));
 	g_print(_("    --debug              Instruct xine to print detailed debug information.\n"));
 	g_print(_("    -f, --fullscreen     Start xfmedia's video window in fullscreen mode.\n"));
-	g_print(_("    -p, --autoplay       Start playing when xfmedia starts.  (This is the\n"
+	g_print(_("    -p, --autoplay       Start playing when Xfmedia starts.  (This is the\n"
 	          "                         default if xfmedia is given filenames on the command\n"
 	          "                         line.)\n"));
+    g_print(_("    -c, --clear-playlist Start xfmedia with an empty playlist.\n"));
 	g_print(_("    --audio-out=DRIVER   Specify the audio output driver to use.\n"));
 	g_print(_("    --video-out=DRIVER   Specify the video output driver to use.\n"));
-	g_print(_("    --session=N          For remote control, specify the number of an already-\n"
-	          "                         running instance of xfmedia.  (Defaults to 0, or the\n"
-	          "                         first instance found.)\n"));
-	g_print(_("    --enqueue            Instructs xfmedia to append any filename arguments to\n"
-	          "                         the playlist of an already-running instance of xfmedia.\n"));
+    g_print(_("    --vwin-geometry=WxH[+X+Y]\n"
+              "                         Specify a fixed size and/or position for xfmedia's\n"
+              "                         video window.\n"));
 }
 
 enum
@@ -244,13 +246,12 @@ enum
     OPT_DISPLAY = 1000,
     OPT_SMCLIENTID,
     OPT_DEBUG,
-    OPT_SESSIONID,
-    OPT_ENQUEUE,
     OPT_AUDIO_OUT,
-    OPT_VIDEO_OUT
+    OPT_VIDEO_OUT,
+    OPT_VWIN_GEOMETRY,
 };
 
-static const gchar *short_opts = "Vhvfp";
+static const gchar *short_opts = "Vhvfpc";
 static const struct option long_opts[] =
 {
     /* stuff we just want out of the way */
@@ -263,11 +264,19 @@ static const struct option long_opts[] =
     { "debug", no_argument, NULL, OPT_DEBUG },
     { "fullscreen", no_argument, NULL, 'f' },
     { "autoplay", no_argument, NULL, 'p' },
-    { "session", required_argument, NULL, OPT_SESSIONID },
-    { "enqueue", no_argument, NULL, OPT_ENQUEUE },
+    { "clear-playlist", no_argument, NULL, 'c' },
     { "audio-out", required_argument, NULL, OPT_AUDIO_OUT },
     { "video-out", required_argument, NULL, OPT_VIDEO_OUT },
-    { NULL, 0, NULL, 0 }
+    { "vwin-geometry", required_argument, NULL, OPT_VWIN_GEOMETRY },
+    { NULL, 0, NULL, 0 },
+};
+
+enum
+{
+    VWIN_GEOM_WIDTH  = 0x0001,
+    VWIN_GEOM_HEIGHT = 0x0002,
+    VWIN_GEOM_X      = 0x0004,
+    VWIN_GEOM_Y      = 0x0008,
 };
 
 typedef struct
@@ -279,6 +288,10 @@ typedef struct
     gint verbose_level;
     gboolean use_fullscreen;
     gboolean use_autoplay;
+    gboolean dont_load_playlist;
+    
+    gint vwin_geom_flags;
+    GdkRectangle vwin_geom;
 } OptionData;
 
 static gint
@@ -298,25 +311,18 @@ xfmedia_parse_options(int argc, char **argv, OptionData *odata)
                 break;
             
             case 'V':
-                g_print("Xfmedia version %s, Copyright (c) Brian Tarricone, <bjt23@cornell.edu>\n", VERSION);
-                g_print("Released under the terms of the GNU General Public License.\n");
-                g_print("Compiled against xine-lib %s, using xine-lib %s.\n",
+                g_print(_("Xfmedia version %s, Copyright (c) Brian Tarricone, <bjt23@cornell.edu>\n"), VERSION);
+                g_print(_("Released under the terms of the GNU General Public License.\n"));
+                g_print(_("Compiled against xine-lib %s, using xine-lib %s.\n"),
                         XINE_VERSION, xine_get_version_string());
-                if((gint)LIBXFCE4UTIL_MICRO_VERSION == LIBXFCE4UTIL_MICRO_VERSION) {
-                    g_print("Compiled against Xfce %d.%d.%d, using Xfce %s.\n",
-                            LIBXFCE4UTIL_MAJOR_VERSION, LIBXFCE4UTIL_MINOR_VERSION,
-                            (gint)LIBXFCE4UTIL_MICRO_VERSION, xfce_version_string());
-                } else {
-                    g_print("Compiled against Xfce %d.%d.%.1f, using Xfce %s.\n",
-                            LIBXFCE4UTIL_MAJOR_VERSION, LIBXFCE4UTIL_MINOR_VERSION,
-                            (gdouble)LIBXFCE4UTIL_MICRO_VERSION, xfce_version_string());
-                }
-                
+                g_print(_("Compiled against Xfce %d.%d.%d, using Xfce %s.\n"),
+                        LIBXFCE4UTIL_MAJOR_VERSION, LIBXFCE4UTIL_MINOR_VERSION,
+                        LIBXFCE4UTIL_MICRO_VERSION, xfce_version_string());
                 exit(0);
                 break;
             
             case 'h':
-                print_usage(argv[0]);
+                xfmedia_print_usage(argv[0]);
                 exit(0);
                 break;
             
@@ -336,21 +342,15 @@ xfmedia_parse_options(int argc, char **argv, OptionData *odata)
                 odata->use_autoplay = TRUE;
                 break;
             
-            case OPT_SESSIONID:
-                odata->is_remote_cmd = TRUE;
-                odata->session_id = atoi(optarg);
-                break;
-            
-            case OPT_ENQUEUE:
-                odata->is_remote_cmd = TRUE;
-                odata->is_enqueue = TRUE;
+            case 'c':
+                odata->dont_load_playlist = TRUE;
                 break;
             
             case OPT_AUDIO_OUT:
                 if(optarg && *optarg)
                     xfmedia_settings_set_string("/xfmedia/plugins/libxine/audio_out", optarg);
                 else {
-                    g_printerr(_("The option --audio-out requires an argument."));
+                    g_printerr(_("The option --audio-out requires an argument.\n"));
                     exit(1);
                 }
                 break;
@@ -359,11 +359,32 @@ xfmedia_parse_options(int argc, char **argv, OptionData *odata)
                 if(optarg && *optarg)
                     xfmedia_settings_set_string("/xfmedia/plugins/libxine/video_out", optarg);
                 else {
-                    g_printerr(_("The option --video-out requires an argument."));
+                    g_printerr(_("The option --video-out requires an argument.\n"));
                     exit(1);
                 }
                 break;
             
+            case OPT_VWIN_GEOMETRY:
+                if(optarg && *optarg) {
+                    guint w, h, x, y;
+                    gint args = sscanf(optarg, "%ux%u+%u+%u", &w, &h, &x, &y);
+                    
+                    odata->vwin_geom_flags = 0;
+                    
+                    if(args >= 2) {
+                        odata->vwin_geom.width = w;
+                        odata->vwin_geom.height = h;
+                        odata->vwin_geom_flags |= VWIN_GEOM_WIDTH|VWIN_GEOM_HEIGHT;
+                    }
+                    
+                    if(args == 4) {
+                        odata->vwin_geom.x = x;
+                        odata->vwin_geom.y = y;
+                        odata->vwin_geom_flags |= VWIN_GEOM_X|VWIN_GEOM_Y;
+                    }
+                }
+                break;
+            
             default:
                 break;
         }
@@ -375,8 +396,8 @@ xfmedia_parse_options(int argc, char **argv, OptionData *odata)
 int
 main(int argc, char **argv)
 {
-    GtkWidget *gtx, *video_window;
-    XfMediaMainwin *mwin;
+    GtkWidget *xfx, *video_window;
+    XfmediaMainwin *mwin;
     gint i, fd;
     gboolean defer = FALSE, do_autoplay = FALSE;
     unsigned int seed;
@@ -385,6 +406,7 @@ main(int argc, char **argv)
     OptionData odata;
     gchar *session_argv[1];
     GdkPixbuf *winicon;
+    const gchar *video_port_id = NULL;
     
     main_pid = getpid();
     
@@ -404,74 +426,7 @@ main(int argc, char **argv)
     xfmedia_settings_init();
     next_arg = xfmedia_parse_options(argc, argv, &odata);
     
-    /* handle remote commands */
-    if(odata.is_remote_cmd) {
-		XfMediaRemoteStatus remote_ret;
-		
-        if(odata.is_enqueue && next_arg < argc) {
-            
-            if(odata.session_id == -1)
-                odata.session_id = xfmedia_remote_send_command(-1, XFMEDIA_REMOTE_IS_RUNNING);
-            else {
-                if(xfmedia_remote_send_command(odata.session_id, XFMEDIA_REMOTE_IS_RUNNING) == XFMEDIA_REMOTE_COMMAND_FAILED)
-                    odata.session_id = xfmedia_remote_send_command(-1, XFMEDIA_REMOTE_IS_RUNNING);
-            }
-            
-            if(odata.session_id == XFMEDIA_REMOTE_COMMAND_FAILED) {
-                DBG("attempting to launch %s", argv[0]);
-                if(xfce_exec(argv[0], FALSE, FALSE, NULL)) {
-                    while((odata.session_id = xfmedia_remote_send_command(-1, XFMEDIA_REMOTE_IS_RUNNING)) == XFMEDIA_REMOTE_COMMAND_FAILED)
-                        ;  /* spin! */
-                } else {
-                    g_printerr(_("Failed to contact or start an instance of Xfmedia.\n"));
-                    return 1;
-                }
-			}
-
-            if(odata.session_id == XFMEDIA_REMOTE_NO_SESSION_BUS) {
-                g_printerr(_("Unable to contact D-BUS session bus.  Be sure the session bus is running."));
-                return 1;
-            } else {
-				DBG("found session %d", odata.session_id);
-            }
-            
-            if(!strstr(argv[next_arg], ":/") && !g_path_is_absolute(argv[next_arg])) {
-                gchar *newfile = g_strdup_printf("%s/%s", g_get_current_dir(),
-                        argv[next_arg]);
-                remote_ret = xfmedia_remote_send_command(odata.session_id,
-                        XFMEDIA_REMOTE_ADD_FILE, newfile, -1);
-                g_free(newfile);
-            } else {
-                remote_ret = xfmedia_remote_send_command(odata.session_id,
-                        XFMEDIA_REMOTE_ADD_FILE, argv[next_arg], -1);
-            }
-            next_arg++;
-			if(remote_ret < XFMEDIA_REMOTE_COMMAND_SUCCEEDED) {
-				g_printerr(_("Failed to contact or start an instance of Xfmedia.\n"));
-				return 1;
-			}
-            while(next_arg < argc) {
-                if(!strstr(argv[next_arg], ":/") && !g_path_is_absolute(argv[next_arg])) {
-                    gchar *newfile = g_strdup_printf("%s/%s",
-                            g_get_current_dir(), argv[next_arg]);
-                    remote_ret = xfmedia_remote_send_command(odata.session_id,
-                            XFMEDIA_REMOTE_ADD_FILE, newfile, -1);
-                    g_free(newfile);
-                } else {
-                    remote_ret = xfmedia_remote_send_command(odata.session_id,
-                            XFMEDIA_REMOTE_ADD_FILE, argv[next_arg], -1);
-                }
-                next_arg++;
-            }
-			if(odata.use_autoplay)
-				xfmedia_remote_send_command(odata.session_id, XFMEDIA_REMOTE_PLAY, remote_ret);
-        } else {
-            /* implement other remote commands here */
-        }
-        
-        return 0;
-    }
-    
+    /* seed PRNG */
     fd = open("/dev/random", O_RDONLY);
     if(fd >= 0) {
         read(fd, &seed, sizeof(unsigned int));
@@ -494,6 +449,13 @@ main(int argc, char **argv)
     gdk_threads_init();
     gtk_init(&argc, &argv);
     
+    DBG("calling XInitThreads()");
+    if(!XInitThreads()) {
+        g_critical("XfmediaXine: XInitThreads() failed; thread-safe X11 libs unavailable!\n");
+        video_port_id = "none";
+    } else
+        video_port_id = xfmedia_settings_get_string("/xfmedia/plugins/libxine/video_out");
+    
     winicon = gdk_pixbuf_from_pixdata(&xfmedia_icon, TRUE, NULL);
     if(winicon) {
         gtk_window_set_default_icon(winicon);
@@ -510,19 +472,35 @@ main(int argc, char **argv)
     gtk_window_set_title(GTK_WINDOW(video_window), _("Xfmedia Video"));
     gtk_container_set_border_width(GTK_CONTAINER(video_window), 0);
     gtk_widget_realize(video_window);
+    if(odata.vwin_geom_flags & (VWIN_GEOM_X|VWIN_GEOM_Y)) {
+        gtk_window_move(GTK_WINDOW(video_window),
+                        odata.vwin_geom.x,
+                        odata.vwin_geom.y);
+    }
+    if(odata.vwin_geom_flags & (VWIN_GEOM_WIDTH|VWIN_GEOM_HEIGHT)) {
+        gtk_widget_set_size_request(video_window,
+                                    odata.vwin_geom.width,
+                                    odata.vwin_geom.height);
+        gtk_window_resize(GTK_WINDOW(video_window),
+                          odata.vwin_geom.width,
+                          odata.vwin_geom.height);
+        g_object_set_data(G_OBJECT(video_window),
+                          "vwin-size-fixed",
+                          GINT_TO_POINTER(0xfeedface));
+    }
     
-    gtx = gtk_xine_new(xfmedia_settings_get_string("/xfmedia/plugins/libxine/video_out"),
+    xfx = xfmedia_xine_new(video_port_id,
             xfmedia_settings_get_string("/xfmedia/plugins/libxine/audio_out"));
-    gtk_widget_show(gtx);
-    gtk_container_add(GTK_CONTAINER(video_window), gtx);
+    gtk_widget_show(xfx);
+    gtk_container_add(GTK_CONTAINER(video_window), xfx);
     DBG("about to realize...");
-    gtk_widget_realize(gtx);
+    gtk_widget_realize(xfx);
     DBG("    realize done");
     
-    gtk_xine_engine_set_param(GTK_XINE(gtx),
+    xfmedia_xine_engine_set_param(XFMEDIA_XINE(xfx),
             XINE_ENGINE_PARAM_VERBOSITY, odata.verbose_level);
     
-    mwin = g_mwin = xfmedia_mainwin_new(GTK_XINE(gtx), GTK_WINDOW(video_window));
+    mwin = g_mwin = xfmedia_mainwin_new(XFMEDIA_XINE(xfx), GTK_WINDOW(video_window));
     mwin_dest_id = g_signal_connect(G_OBJECT(mwin->window), "destroy",
             G_CALLBACK(mainwin_destroy_cb), mwin);
     vwin_dest_id = g_signal_connect(G_OBJECT(video_window), "destroy",
@@ -542,15 +520,10 @@ main(int argc, char **argv)
         do_autoplay = TRUE;
     }
     
-    if(next_arg >= argc)
+    if(next_arg >= argc && !odata.dont_load_playlist)
         xfmedia_settings_load_playlist(mwin);
     
-#if 0
-    /* set up the shuffle bitmap.  this is somewhat redundant, depending, but
-     * i want to make sure it gets done properly. (FIXME: startup perf) */
-    xfmedia_playlist_shuffle_init_entries(mwin->plist,
-            gtk_tree_model_iter_n_children(GTK_TREE_MODEL(mwin->plist->file_list), NULL));
-#endif
+    do_startup_notify(mwin->window);
     
     x = xfmedia_settings_get_int("/xfmedia/general/pos_x");
     y = xfmedia_settings_get_int("/xfmedia/general/pos_y");
@@ -558,8 +531,8 @@ main(int argc, char **argv)
         gtk_window_move(GTK_WINDOW(mwin->window), x, y);
     gtk_widget_show(mwin->window);
     gtk_widget_set_size_request(mwin->window, -1, -1);
-    
-    do_startup_notify(mwin->window);
+    if(xfmedia_settings_get_bool("/xfmedia/general/dock_autohide"))
+        xfmedia_mainwin_set_autohide(mwin, TRUE);
     
     session_argv[0] = argv[0];
     session_client = exo_xsession_client_new_with_group(mwin->window->window);
@@ -585,7 +558,7 @@ main(int argc, char **argv)
     xfmedia_plugins_load(mwin);
     
     if(odata.use_fullscreen)
-        gtk_xine_set_fullscreen(mwin->gtx, TRUE);
+        xfmedia_xine_set_fullscreen(mwin->xfx, TRUE);
     if(do_autoplay || odata.use_autoplay)
         g_idle_add(initial_autoplay, mwin);
     
