@@ -22,7 +22,7 @@
 
 #include <glib.h>
 
-#include "xfmedia-common.h"
+#include "mainwin.h"
 
 G_BEGIN_DECLS
 
@@ -30,9 +30,9 @@ typedef enum {
 	XFMEDIA_QUIT_MODE_EXT = 0,
 	XFMEDIA_QUIT_MODE_MAINWIN_DESTROY,
 	XFMEDIA_QUIT_MODE_VWIN_DESTROY
-} XfMediaQuitMode;
+} XfmediaQuitMode;
 
-void xfmedia_quit(XfMediaMainwin *mwin, XfMediaQuitMode quit_mode);
+void xfmedia_quit(XfmediaMainwin *mwin, XfmediaQuitMode quit_mode);
 
 G_END_DECLS
 
