@@ -90,7 +90,7 @@ typedef struct
     GNode *cur_folder;
     gint next_index;
     gboolean started;
-} XfMediaMMState;
+} XfmediaMMState;
 
 static GNode *mediamarks_root = NULL;
 static GList *mm_changed_cb = NULL;
@@ -121,8 +121,13 @@ mm_write_all()
     mmfile = xfce_resource_save_location(XFCE_RESOURCE_CONFIG, MEDIAMARKS_FILE,
             TRUE);
     if(mmfile) {
-        if(!xfmedia_mediamarks_save(mmfile))
-            g_critical("Unable to save mediamarks to file %s", mmfile);
+        if(!xfmedia_mediamarks_save(mmfile)) {
+            xfce_message_dialog(NULL, _("Mediamarks Save Error"),
+                    GTK_STOCK_DIALOG_ERROR,
+                    _("The mediamarks file failed to save."),
+                    _("This could be because your disk is full, or you don't have permission to write to the configuration directory.  Any changes will be lost when you quit Xfmedia."),
+                    GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
+        }
         g_free(mmfile);
     }
 }
@@ -130,7 +135,7 @@ mm_write_all()
 static gboolean
 mediamarks_free_all(GNode *node, gpointer data)
 {
-    XfMediaMediamark *mmark = node->data;
+    XfmediaMediamark *mmark = node->data;
     
     if(!mmark)
         return FALSE;
@@ -151,8 +156,8 @@ mediamarks_xml_start(GMarkupParseContext *context, const gchar *element_name,
         const gchar **attribute_names, const gchar **attribute_values,
         gpointer user_data, GError **error)
 {
-    XfMediaMMState *state = user_data;
-    XfMediaMediamark *mmark;
+    XfmediaMMState *state = user_data;
+    XfmediaMediamark *mmark;
     GNode *node;
     
     if(!state->started) {
@@ -163,9 +168,9 @@ mediamarks_xml_start(GMarkupParseContext *context, const gchar *element_name,
     
     if(!strcmp(element_name, "folder")) {
         if(attribute_names[0] && !strcmp(attribute_names[0], "name")) {
-            mmark = g_new0(XfMediaMediamark, 1);
+            mmark = g_new0(XfmediaMediamark, 1);
             mmark->name = g_strdup(attribute_values[0]);
-            mmark->index = state->next_index;
+            mmark->idx = state->next_index;
             node = g_node_new(mmark);
             g_node_append(state->cur_folder, node);
             
@@ -184,10 +189,10 @@ mediamarks_xml_start(GMarkupParseContext *context, const gchar *element_name,
         }
         
         if(name && uri) {
-            mmark = g_new0(XfMediaMediamark, 1);
+            mmark = g_new0(XfmediaMediamark, 1);
             mmark->name = g_strdup(name);
             mmark->uri = g_strdup(uri);
-            mmark->index = state->next_index++;
+            mmark->idx = state->next_index++;
             node = g_node_new(mmark);
             g_node_append(state->cur_folder, node);
         }
@@ -198,12 +203,12 @@ static void
 mediamarks_xml_end(GMarkupParseContext *context, const gchar *element_name,
         gpointer user_data, GError **error)
 {
-    XfMediaMMState *state = user_data;
+    XfmediaMMState *state = user_data;
     
     if(!strcmp(element_name, "xfmedia-mediamarks"))
         state->started = FALSE;
     else if(!strcmp(element_name, "folder")) {
-        state->next_index = ((XfMediaMediamark *)state->cur_folder->data)->index + 1;
+        state->next_index = ((XfmediaMediamark *)state->cur_folder->data)->idx + 1;
         if(!G_NODE_IS_ROOT(state->cur_folder))
             state->cur_folder = state->cur_folder->parent;
     }
@@ -213,9 +218,11 @@ static void
 mediamarks_write_xml_tree(FILE *fp, GNode *node, gint depth)
 {
     gchar tabs[128], *name_esc, *uri_esc = NULL;
-    XfMediaMediamark *mmark;
+    XfmediaMediamark *mmark;
     GNode *child;
     
+    g_return_if_fail(node);
+    
     if(depth > 127)
         depth = 127;
     memset(tabs, '\t', depth);
@@ -253,31 +260,31 @@ mediamarks_node_is_in_tree(GNode *node)
 
 static GNode *
 mediamarks_insert_node(GNode *parent, const gchar *name, const gchar *uri,
-        gint index)
+        gint idx)
 {
-    XfMediaMediamark *mmark;
+    XfmediaMediamark *mmark;
     GNode *node = NULL;
     
-    mmark = g_new0(XfMediaMediamark, 1);
+    mmark = g_new0(XfmediaMediamark, 1);
     mmark->name = g_strdup(name);
     if(uri)
         mmark->uri = g_strdup(uri);
-    if(index < 0)
-        mmark->index = g_node_n_children(parent);
+    if(idx < 0)
+        mmark->idx = g_node_n_children(parent);
     
     node = g_node_new(mmark);
-    if(index < 0)
+    if(idx < 0)
         g_node_append(parent, node);
     else {
         gint i;
         
-        g_node_insert(parent, index, node);
+        g_node_insert(parent, idx, node);
         
-        for(i = index + 1; ; i++) {
+        for(i = idx + 1; ; i++) {
             GNode *node = g_node_nth_child(parent, i);
             if(!node)
                 break;
-            ((XfMediaMediamark *)node->data)->index = i;
+            ((XfmediaMediamark *)node->data)->idx = i;
         }
     }
     
@@ -288,13 +295,13 @@ void
 xfmedia_mediamarks_init()
 {
     gchar *mmfile;
-    XfMediaMediamark *mmark;
+    XfmediaMediamark *mmark;
     
     g_return_if_fail(!mediamarks_root);
     
-    mmark = g_new0(XfMediaMediamark, 1);
+    mmark = g_new0(XfmediaMediamark, 1);
     mmark->name = g_strdup("/");
-    mmark->index = -1;
+    mmark->idx = -1;
     mediamarks_root = g_node_new(mmark);
     
     mmfile = xfce_resource_save_location(XFCE_RESOURCE_CONFIG, MEDIAMARKS_FILE,
@@ -302,9 +309,9 @@ xfmedia_mediamarks_init()
     if(mmfile) {
         if(!xfmedia_mediamarks_load(mmfile)) {
             xfmedia_mediamarks_cleanup();
-            mmark = g_new0(XfMediaMediamark, 1);
+            mmark = g_new0(XfmediaMediamark, 1);
             mmark->name = g_strdup("/");
-            mmark->index = -1;
+            mmark->idx = -1;
             mediamarks_root = g_node_new(mmark);
         }
         g_free(mmfile);
@@ -340,7 +347,7 @@ xfmedia_mediamarks_load(const gchar *filename)
         mediamarks_xml_start, mediamarks_xml_end, NULL, NULL, NULL
     };
     GMarkupParseContext *pctx;
-    XfMediaMMState state;
+    XfmediaMMState state;
     GError *err = NULL;
     
     if(stat(filename, &st))
@@ -355,8 +362,14 @@ xfmedia_mediamarks_load(const gchar *filename)
         file_contents = addr;
 #endif
     
-    if(!file_contents)
-        g_file_get_contents(filename, &file_contents, &file_length, NULL);
+    if(!file_contents) {
+        if(!g_file_get_contents(filename, &file_contents, &file_length, NULL)) {
+#if HAVE_SYS_MMAN_H
+            close(fd);
+#endif
+            return FALSE;
+        }
+    }
     
     state.cur_folder = mediamarks_root;
     state.next_index = 0;
@@ -415,28 +428,8 @@ xfmedia_mediamarks_save(const gchar *filename)
     ret = TRUE;
     fclose(fp);
     
-    if(g_file_test(filename, G_FILE_TEST_EXISTS)) {
-        if(!unlink(filename)) {
-            if(!link(mmfile_new, filename)) {
-                if(unlink(mmfile_new))
-                    g_warning("Unable to delete temp file when writing mediamarks");
-            } else {
-                g_warning("Unable to copy temp file to mediamarks file");
-                ret = FALSE;
-            }
-        } else {
-            g_warning("Unable to remove old mediamarks file");
-            ret = FALSE;
-        }
-    } else {
-        if(!link(mmfile_new, filename)) {
-            if(unlink(mmfile_new))
-                g_warning("Unable to delete temp file when writing mediamarks");
-        } else {
-            g_warning("Unable to copy temp file to mediamarks file");
-            ret = FALSE;
-        }
-    }
+    if(!rename(mmfile_new, filename))
+        ret = TRUE;
     
     cleanup:
     
@@ -447,14 +440,14 @@ xfmedia_mediamarks_save(const gchar *filename)
 }
 
 GNode *
-xfmedia_mediamarks_insert_folder(GNode *parent, const gchar *name, gint index)
+xfmedia_mediamarks_insert_folder(GNode *parent, const gchar *name, gint idx)
 {
     GNode *node;
     
     g_return_val_if_fail(parent && name
             && mediamarks_node_is_in_tree(parent), NULL);
     
-    node = mediamarks_insert_node(parent, name, NULL, index);
+    node = mediamarks_insert_node(parent, name, NULL, idx);
     
     mm_changed_notify();
     mm_write_all();
@@ -466,7 +459,7 @@ gboolean
 xfmedia_mediamarks_remove_folder(GNode *folder_node)
 {
     g_return_val_if_fail(folder_node
-            && !((XfMediaMediamark *)folder_node->data)->uri
+            && !((XfmediaMediamark *)folder_node->data)->uri
             && mediamarks_node_is_in_tree(folder_node), FALSE);
     
     g_node_traverse(folder_node, G_IN_ORDER, G_TRAVERSE_ALL, -1,
@@ -482,14 +475,14 @@ xfmedia_mediamarks_remove_folder(GNode *folder_node)
 
 GNode *
 xfmedia_mediamarks_insert_mark(GNode *parent, const gchar *name,
-        const gchar *uri, gint index)
+        const gchar *uri, gint idx)
 {
     GNode *node;
     
     g_return_val_if_fail(parent && name && uri
             && mediamarks_node_is_in_tree(parent), FALSE);
     
-    node = mediamarks_insert_node(parent, name, uri, index);
+    node = mediamarks_insert_node(parent, name, uri, idx);
     
     mm_changed_notify();
     mm_write_all();
@@ -501,7 +494,7 @@ gboolean
 xfmedia_mediamarks_remove_mark(GNode *mmark_node)
 {
     g_return_val_if_fail(mmark_node
-            && ((XfMediaMediamark *)mmark_node->data)->uri
+            && ((XfmediaMediamark *)mmark_node->data)->uri
             && mediamarks_node_is_in_tree(mmark_node), FALSE);
     
     g_node_unlink(mmark_node);
@@ -615,7 +608,7 @@ static gboolean
 mmtree_add_folders(GNode *node, gpointer data)
 {
     GtkTreeStore *ts = data;
-    XfMediaMediamark *mmark = node->data;
+    XfmediaMediamark *mmark = node->data;
     static gint parent_depth = 1;
     static GtkTreeIter itr, pitr;
     
@@ -661,7 +654,7 @@ mmtree_add_folders(GNode *node, gpointer data)
 }
 
 static GtkWidget *
-mm_create_folder_treeview(XfMediaMainwin *mwin, GtkBox *box)
+mm_create_folder_treeview(XfmediaMainwin *mwin, GtkBox *box)
 {
     GtkWidget *treeview, *sw;
     GtkTreeStore *ts;
@@ -712,7 +705,7 @@ mm_create_folder_treeview(XfMediaMainwin *mwin, GtkBox *box)
 }
 
 GNode *
-xfmedia_mediamarks_add(XfMediaMainwin *mwin, GtkWindow *parent,
+xfmedia_mediamarks_add(XfmediaMainwin *mwin, GtkWindow *parent,
         const gchar *title, const gchar *uri)
 {
     GtkTreePath *path;
@@ -850,12 +843,12 @@ xfmedia_mediamarks_add(XfMediaMainwin *mwin, GtkWindow *parent,
 }
 
 static void
-xfmedia_mediamarks_add_index(XfMediaMainwin *mwin, GtkWindow *parent,
-        gint index)
+xfmedia_mediamarks_add_index(XfmediaMainwin *mwin, GtkWindow *parent,
+        gint idx)
 {
     gchar *title = NULL, *uri = NULL;
     
-    if(xfmedia_playlist_get(mwin->plist, index, &title, NULL, &uri)) {
+    if(xfmedia_playlist_get(mwin->plist, idx, &title, NULL, &uri)) {
         xfmedia_mediamarks_add(mwin, parent, title, uri);
         g_free(title);
         g_free(uri);
@@ -865,22 +858,22 @@ xfmedia_mediamarks_add_index(XfMediaMainwin *mwin, GtkWindow *parent,
 static void
 mm_add_cur_stream_cb(GtkWidget *w, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
-    gint index;
+    XfmediaMainwin *mwin = user_data;
+    gint idx;
     
     if(!mwin->cur_playing)
         return;
     
-    index = xfmedia_playlist_entry_ref_get_index(mwin->cur_playing);
-    if(index != -1)
-        xfmedia_mediamarks_add_index(mwin, GTK_WINDOW(mwin->window), index);
+    idx = xfmedia_playlist_entry_ref_get_index(mwin->cur_playing);
+    if(idx != -1)
+        xfmedia_mediamarks_add_index(mwin, GTK_WINDOW(mwin->window), idx);
 }
 
 static gboolean
 mmtree_add_all(GNode *node, gpointer data)
 {
     GtkTreeStore *ts = data;
-    XfMediaMediamark *mmark = node->data;
+    XfmediaMediamark *mmark = node->data;
     static gint parent_depth = 1;
     static GtkTreeIter itr, pitr;
     
@@ -937,7 +930,7 @@ mmtree_add_all(GNode *node, gpointer data)
 
 typedef struct
 {
-    XfMediaMainwin *mwin;
+    XfmediaMainwin *mwin;
     GtkWidget *treeview;
 } MMManagerData;
 
@@ -1043,7 +1036,7 @@ mmedit_rmitem_cb(GtkWidget *w, gpointer user_data)
     {
         GtkWidget *toplevel = gtk_widget_get_toplevel(mdata->treeview);
         GNode *item_node = NULL;
-        XfMediaMediamark *mmark;
+        XfmediaMediamark *mmark;
         gint resp;
         
         gtk_tree_model_get(ts, &itr, MMEDIT_COL_NODE, &item_node, -1);
@@ -1172,7 +1165,7 @@ mmedit_props_cb(GtkWidget *w, gpointer user_data)
         GtkWindow *parent = GTK_WINDOW(gtk_widget_get_toplevel(mdata->treeview));
         GtkSizeGroup *sg;
         GNode *node = NULL;
-        XfMediaMediamark *mmark;
+        XfmediaMediamark *mmark;
         
         gtk_tree_model_get(ts, &itr, MMEDIT_COL_NODE, &node, -1);
         mmark = node->data;
@@ -1278,7 +1271,7 @@ mmedit_btnpress_cb(GtkWidget *w, GdkEventButton *evt, gpointer user_data)
         
         if(gtk_tree_selection_get_selected(sel, &ts, &itr)) {
             GNode *node = NULL;
-            XfMediaMediamark *mmark;
+            XfmediaMediamark *mmark;
             
             gtk_tree_model_get(ts, &itr, MMEDIT_COL_NODE, &node, -1);
             mmark = node->data;
@@ -1290,7 +1283,7 @@ mmedit_btnpress_cb(GtkWidget *w, GdkEventButton *evt, gpointer user_data)
                     gtk_tree_view_collapse_row(GTK_TREE_VIEW(mdata->treeview), path);
                 gtk_tree_path_free(path);
             } else
-                xfmedia_mainwin_play_uri(mdata->mwin, mmark->uri);
+                xfmedia_mainwin_play_uri(mdata->mwin, mmark->uri, -1);
         }
     } else if(evt->button == 3) {
         GtkWidget *menu, *mi;
@@ -1420,13 +1413,13 @@ mmedit_treeview_drag_rcv_cb(GtkWidget *widget, GdkDragContext *drag_context,
                 GNode *src_node = NULL, *dest_node = NULL;
                 GtkTreeSelection *sel;
 				gboolean dest_is_folder;
-				XfMediaMediamark *mmark;
+				XfmediaMediamark *mmark;
 				
                 gtk_tree_model_get(ts, &dest_itr, MMEDIT_COL_NODE, &dest_node, -1);
 				
 				if(dest_node && src_node && dest_node->parent && src_node->parent) {
 					mmark = src_node->data;
-					dest_is_folder = !((XfMediaMediamark *)dest_node->data)->uri;
+					dest_is_folder = !((XfmediaMediamark *)dest_node->data)->uri;
 					
 					g_node_unlink(src_node);
 					
@@ -1466,7 +1459,7 @@ mmedit_treeview_drag_rcv_cb(GtkWidget *widget, GdkDragContext *drag_context,
 #if 0  /* perhaps i'll eventually implement dropping files from a FM */
     else if(data->target == _URI_LIST) {
         GList *uris, *l;
-        gint from_index = -1, index, num_added = 0;
+        gint from_index = -1, idx, num_added = 0;
         
         DBG("we have an uri list: '%s'", (gchar *)data->data);
         
@@ -1487,7 +1480,7 @@ mmedit_treeview_drag_rcv_cb(GtkWidget *widget, GdkDragContext *drag_context,
             gtk_tree_path_free(dest_path);
         }
         
-        index = from_index;
+        idx = from_index;
         uris = xfmedia_uri_list_extract_uris((const gchar *)data->data);
         for(l = uris; l; l = l->next) {
             gchar *filename = l->data;
@@ -1495,10 +1488,10 @@ mmedit_treeview_drag_rcv_cb(GtkWidget *widget, GdkDragContext *drag_context,
             if(!g_ascii_strncasecmp(filename, "file://", 7))
                 filename += 7;
             DBG("adding '%s'", filename);
-            xfmedia_mainwin_add_file(mwin, filename, index, TRUE);
+            xfmedia_mainwin_add_file(mwin, filename, idx, TRUE);
             g_free(l->data);
-            if(index != -1)
-                index++;
+            if(idx != -1)
+                idx++;
             num_added++;
         }
         if(uris) {
@@ -1574,7 +1567,7 @@ mm_manager_zero_pointer(GtkWidget **w)
 static void
 manage_mediamarks_cb(GtkWidget *w, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     static GtkWidget *win = NULL;
     GtkWidget *topvbox, *vbox, *hbox, *toolbar, *img, *sw, *treeview, *btn;
     GtkTreeStore *ts;
@@ -1761,21 +1754,21 @@ manage_mediamarks_cb(GtkWidget *w, gpointer user_data)
 static void
 mm_item_activate_cb(GtkWidget *w, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
-    XfMediaMediamark *mmark;
+    XfmediaMainwin *mwin = user_data;
+    XfmediaMediamark *mmark;
     
     mmark = g_object_get_data(G_OBJECT(w), "xfmedia-mmark");
     if(!mmark || !mmark->uri)
         return;
     
-    xfmedia_mainwin_play_uri(mwin, mmark->uri);
+    xfmedia_mainwin_play_uri(mwin, mmark->uri, -1);
 }
 
 static void
-mm_generate_menu(XfMediaMainwin *mwin, GtkMenuShell *menu, GNode *root)
+mm_generate_menu(XfmediaMainwin *mwin, GtkMenuShell *menu, GNode *root)
 {
     GNode *child;
-    XfMediaMediamark *mmark;
+    XfmediaMediamark *mmark;
     GtkWidget *mi, *img, *menu2;
     
     mmark = root->data;
@@ -1802,7 +1795,7 @@ mm_generate_menu(XfMediaMainwin *mwin, GtkMenuShell *menu, GNode *root)
 }
 
 GtkWidget *
-xfmedia_mediamarks_create_menu(XfMediaMainwin *mwin)
+xfmedia_mediamarks_create_menu(XfmediaMainwin *mwin)
 {
     GtkWidget *menu, *mi, *img;
     GNode *root, *child;
