@@ -20,14 +20,30 @@
 #ifndef __XFMEDIA_PLAYLIST_FILES_H__
 #define __XFMEDIA_PLAYLIST_FILES_H__
 
-#include "xfmedia-common.h"
+#include <xfmedia/xfmedia-playlist.h>
 
 G_BEGIN_DECLS
 
-gboolean xfmedia_playlists_is_playlist_file(const gchar *filename);
+typedef enum
+{
+    XFMEDIA_PLAYLIST_FORMAT_UNKNOWN = 0,
+    XFMEDIA_PLAYLIST_FORMAT_FLAT_FILE,
+    XFMEDIA_PLAYLIST_FORMAT_M3U,
+    XFMEDIA_PLAYLIST_FORMAT_PLS,
+    XFMEDIA_PLAYLIST_FORMAT_ASX,
+    XFMEDIA_PLAYLIST_FORMAT_WAX
+} XfmediaPlaylistFormat;
 
-gboolean xfmedia_playlists_load(XfMediaPlaylist *plist, const gchar *filename);
-gboolean xfmedia_playlists_save(XfMediaPlaylist *plist, const gchar *filename);
+gboolean xfmedia_playlists_is_playlist_file(const gchar *filename);
+XfmediaPlaylistFormat xfmedia_playlists_query_playlist_format(const gchar *filename);
+
+gboolean xfmedia_playlists_load(XfmediaPlaylist *plist, const gchar *filename);
+gboolean xfmedia_playlists_save(XfmediaPlaylist *plist, const gchar *filename, XfmediaPlaylistFormat format);
+
+gchar *xfmedia_playlists_get_first_entry(const gchar *filename, gchar **title, gint *stream_length);
+
+gchar *xfmedia_playlists_cache_find(const gchar *original_uri);
+gchar *xfmedia_playlists_cache_save(const gchar *original_uri, const gchar *data, const gchar *extension);
 
 G_END_DECLS
 
