@@ -33,47 +33,10 @@
 #include <stdlib.h>
 #endif
 
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
-
-#ifdef HAVE_SYS_UN_H
-#include <sys/un.h>
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-
-#ifdef HAVE_ERRNO_H
-#include <errno.h>
-#endif
-
-#if defined(HAVE_STDARG_H)
-#include <stdarg.h>
-#elif defined(HAVE_VARARGS_H)
-#include <varargs.h>
-#define va_start(v, l) va_start(v)
-#else
-#error Your system does not support variable-argument functions.
-#endif
-
-#ifndef PATH_MAX
-#define PATH_MAX 4096
-#endif
-
-#ifndef UNIX_PATH_MAX
-#define UNIX_PATH_MAX 108
-#endif
-
 #ifdef HAVE_DBUS
 
 #ifndef DBUS_API_SUBJECT_TO_CHANGE  /* exo 0.2 sets this, but 0.3 doesn't */
@@ -94,12 +57,8 @@
 
 #endif  /* HAVE_DBUS */
 
-
 #include <libxfce4util/libxfce4util.h>
 
-#define EXO_API_SUBJECT_TO_CHANGE
-#include <exo/exo.h>
-
 #include "remote.h"
 #include <xfmedia/xfmedia-playlist.h>
 #include "mainwin.h"
@@ -108,17 +67,11 @@
 #include "playlist-files.h"
 #include "xfmedia-common.h"
 
-#define MAX_INSTANCES             1024
-#define XFMEDIA_DBUS_VERSION      "1"
-#define XFMEDIA_DBUS_BASENAME     "org.spuriousinterrupt.Xfmedia"
-#define XFMEDIA_DBUS_INTERFACE    XFMEDIA_DBUS_BASENAME XFMEDIA_DBUS_VERSION
-#define XFMEDIA_DBUS_SERVICE_FMT  XFMEDIA_DBUS_BASENAME "%d"
-#define XFMEDIA_DBUS_ERROR        XFMEDIA_DBUS_BASENAME XFMEDIA_DBUS_VERSION ".Error"
-#define XFMEDIA_DBUS_PATH_FMT     "/org/spuriousinterrupt/Xfmedia/%d"
-/*#define XFMEDIA_DBUS_PATH_FMT     XFMEDIA_DBUS_PARENT_PATH "/%d"*/
-
 #ifdef HAVE_DBUS
 
+extern DBusConnection *__xfmedia_dbus_get_bus_connection();
+
+
 static void xfmedia_dbus_unregister(DBusConnection *connection, void *user_data);
 static DBusHandlerResult xfmedia_dbus_message(DBusConnection *connection, DBusMessage *message, void *user_data);
 
@@ -129,30 +82,10 @@ static const struct DBusObjectPathVTable xfmedia_dbus_vtable =
     NULL,
 };
 
-static DBusGConnection *__dbus_connection = NULL;
-
-static DBusConnection *
-xfmedia_dbus_get_bus_connection()
-{
-    GError *err = NULL;
-    
-    if(!__dbus_connection) {
-        __dbus_connection = dbus_g_bus_get(DBUS_BUS_SESSION, &err);
-        if(!__dbus_connection) {
-            g_warning("Failed to open a connection to the D-BUS session bus.  "
-                    "Please ensure that the session bus is running. (%d: %s)",
-                    err->code, err->message);
-            g_error_free(err);
-        }
-    }
-    
-    return dbus_g_connection_get_connection(__dbus_connection);
-}
-
 static void
 xfmedia_dbus_unregister(DBusConnection *connection, void *user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     
     if(mwin->session_id == -1)
         return;
@@ -164,7 +97,7 @@ static DBusHandlerResult
 xfmedia_dbus_message(DBusConnection *connection, DBusMessage *message,
         void *user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     DBusHandlerResult ret = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     DBusMessage *msg_ret = NULL;
     DBusError derr;
@@ -225,7 +158,7 @@ xfmedia_dbus_message(DBusConnection *connection, DBusMessage *message,
                     DBUS_TYPE_INVALID)
                 && filename && *filename)
         {
-            if(xfmedia_playlists_save(mwin->plist, filename))
+            if(xfmedia_playlists_save(mwin->plist, filename, XFMEDIA_PLAYLIST_FORMAT_UNKNOWN))
                 msg_ret = dbus_message_new_method_return(message);
             g_free(filename);
         } else
@@ -238,33 +171,104 @@ xfmedia_dbus_message(DBusConnection *connection, DBusMessage *message,
         dbus_connection_send(connection, msg_ret, NULL);
         dbus_message_unref(msg_ret);
         
+        ret = DBUS_HANDLER_RESULT_HANDLED;
+    } else if(dbus_message_is_method_call(message, XFMEDIA_DBUS_INTERFACE,
+            XFMEDIA_REMOTE_GET_PLAYLIST_ENTRY))
+    {
+        guint idx;
+
+        dbus_error_init(&derr);
+        if(dbus_message_get_args(message, &derr,
+                    DBUS_TYPE_UINT32, &idx,
+                    DBUS_TYPE_INVALID))
+        {
+            gchar *title = NULL, *filename = NULL;
+            gint length = -1;
+            
+            if(xfmedia_playlist_get(mwin->plist, idx, &title, &length, &filename)) {
+                if(!title)
+                    title = g_strdup(_("(no title)"));
+                msg_ret = dbus_message_new_method_return(message);
+                dbus_message_append_args(msg_ret,
+#ifdef DBUS_USE_OLD_API
+                        DBUS_TYPE_STRING, title,
+                        DBUS_TYPE_INT32, length,
+                        DBUS_TYPE_STRING, filename,
+#else
+                        DBUS_TYPE_STRING, &title,
+                        DBUS_TYPE_INT32, &length,
+                        DBUS_TYPE_STRING, &filename,
+#endif
+                        DBUS_TYPE_INVALID);
+            }
+            g_free(title);
+            g_free(filename);
+        } else
+            dbus_error_free(&derr);
+        
+        if(!msg_ret)
+            msg_ret = dbus_message_new_error(message, XFMEDIA_DBUS_ERROR,
+                    XFMEDIA_REMOTE_GET_PLAYLIST_ENTRY);
+
+        dbus_connection_send(connection, msg_ret, NULL);
+        dbus_message_unref(msg_ret);
+
+        ret = DBUS_HANDLER_RESULT_HANDLED;
+    } else if(dbus_message_is_method_call(message, XFMEDIA_DBUS_INTERFACE,
+            XFMEDIA_REMOTE_GET_PLAYLIST))
+    {
+        gchar **strv = NULL;
+        gchar *cur = NULL;
+        guint nentries, i;
+        
+        nentries = xfmedia_playlist_get_n_entries(mwin->plist);
+        strv = g_new(gchar *, nentries + 1);
+        for(i = 0; i < nentries; i++) {
+            if(xfmedia_playlist_get(mwin->plist, i, &cur, NULL, NULL))
+                strv[i] = cur;
+        }
+        strv[i] = NULL;
+        
+        msg_ret = dbus_message_new_method_return(message);
+        dbus_message_append_args(msg_ret,
+#ifdef DBUS_USE_OLD_API
+                DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, strv, nentries,
+#else
+                DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &strv, nentries,
+#endif
+                DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, msg_ret, NULL);
+        dbus_message_unref(msg_ret);
+
+        g_strfreev(strv);
+
         ret = DBUS_HANDLER_RESULT_HANDLED;
     } else if(dbus_message_is_method_call(message, XFMEDIA_DBUS_INTERFACE,
             XFMEDIA_REMOTE_ADD_FILE))
     {
         gchar *filename = NULL;
-        gint index = -1;
+        gint idx = -1;
         
         dbus_error_init(&derr);
         if(dbus_message_get_args(message, &derr,
                     DBUS_TYPE_STRING, &filename,
-                    DBUS_TYPE_INT32, &index,
+                    DBUS_TYPE_INT32, &idx,
                     DBUS_TYPE_INVALID)
                 && filename && *filename)
         {
             gchar *name_utf8 = xfmedia_filename_to_name(filename);
-            if(index < 0)
-                index = xfmedia_playlist_append_entry(mwin->plist, name_utf8, -1, filename, FALSE);
+            if(idx < 0)
+                idx = xfmedia_playlist_append_entry(mwin->plist, name_utf8, -1, filename, FALSE);
             else
-                index = xfmedia_playlist_insert_entry(mwin->plist, index, name_utf8, -1, filename, FALSE);
+                idx = xfmedia_playlist_insert_entry(mwin->plist, idx, name_utf8, -1, filename, FALSE);
             g_free(name_utf8);
             
             msg_ret = dbus_message_new_method_return(message);
             dbus_message_append_args(msg_ret,
 #ifdef DBUS_USE_OLD_API
-                    DBUS_TYPE_INT32, index,
+                    DBUS_TYPE_INT32, idx,
 #else
-                    DBUS_TYPE_INT32, &index,
+                    DBUS_TYPE_INT32, &idx,
 #endif
                     DBUS_TYPE_INVALID);
             dbus_connection_send(connection, msg_ret, NULL);
@@ -283,14 +287,14 @@ xfmedia_dbus_message(DBusConnection *connection, DBusMessage *message,
     } else if(dbus_message_is_method_call(message, XFMEDIA_DBUS_INTERFACE,
             XFMEDIA_REMOTE_REMOVE_FILE))
     {
-        gint index = 0;
+        gint idx = 0;
         
         dbus_error_init(&derr);
         if(dbus_message_get_args(message, &derr,
-                DBUS_TYPE_UINT32, &index,
+                DBUS_TYPE_UINT32, &idx,
                 DBUS_TYPE_INVALID))
         {
-            if(xfmedia_playlist_remove_entry(mwin->plist, index))
+            if(xfmedia_playlist_remove_entry(mwin->plist, idx))
                 msg_ret = dbus_message_new_method_return(message);
         } else
             dbus_error_free(&derr);
@@ -306,25 +310,27 @@ xfmedia_dbus_message(DBusConnection *connection, DBusMessage *message,
     } else if(dbus_message_is_method_call(message, XFMEDIA_DBUS_INTERFACE,
             XFMEDIA_REMOTE_PLAY))
     {
-        gint index = -1;
+        gint idx = -1;
         gboolean done = FALSE;
         
         dbus_error_init(&derr);
         if(dbus_message_get_args(message, &derr,
-                DBUS_TYPE_INT32, &index,
+                DBUS_TYPE_INT32, &idx,
                 DBUS_TYPE_INVALID))
         {
-            if(index == -1) {
-                if(gtk_xine_get_status(mwin->gtx) == XINE_STATUS_PLAY
-                            && gtk_xine_get_param(mwin->gtx, XINE_PARAM_SPEED) == XINE_SPEED_PAUSE)
+            if(idx == -1) {
+                if(xfmedia_xine_get_status(mwin->xfx) == XINE_STATUS_PLAY
+                            && xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_SPEED) == XINE_SPEED_PAUSE)
                 {
-                    gtk_xine_set_param(mwin->gtx, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
+                    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
                     done = TRUE;
-                } else
-                    index = xfmedia_playlist_get_selected(mwin->plist);
+                } else if(mwin->cur_playing)
+                    idx = xfmedia_playlist_entry_ref_get_index(mwin->cur_playing);
+                else
+                    idx = xfmedia_playlist_get_selected(mwin->plist);
             }
             
-            if(done || xfmedia_mainwin_play_file_at_index(mwin, index))
+            if(done || xfmedia_mainwin_play_file_at_index(mwin, idx))
                 msg_ret = dbus_message_new_method_return(message);
             else
                 msg_ret = dbus_message_new_error(message, XFMEDIA_DBUS_ERROR, XFMEDIA_REMOTE_PLAY);
@@ -339,11 +345,21 @@ xfmedia_dbus_message(DBusConnection *connection, DBusMessage *message,
         
         ret = DBUS_HANDLER_RESULT_HANDLED;
     } else if(dbus_message_is_method_call(message, XFMEDIA_DBUS_INTERFACE,
-            XFMEDIA_REMOTE_PAUSE))
+            XFMEDIA_REMOTE_TOGGLE_PLAY))
     {
         xfmedia_mainwin_toggle_playpause(mwin);
+    } else if(dbus_message_is_method_call(message, XFMEDIA_DBUS_INTERFACE,
+            XFMEDIA_REMOTE_PAUSE))
+    {
+        /* yeah, xfmedia_mainwin_toggle_playpause() is crappy.  it's used to
+         * start playback from the stopped state as well, which we don't really
+         * want.  so let's fudge it. */
+        if(xfmedia_xine_get_status(mwin->xfx) == XINE_STATUS_PLAY) {
+            xfmedia_mainwin_toggle_playpause(mwin);
+            msg_ret = dbus_message_new_method_return(message);
+        } else
+            msg_ret = dbus_message_new_error(message, XFMEDIA_DBUS_ERROR, XFMEDIA_REMOTE_PAUSE);
         
-        msg_ret = dbus_message_new_method_return(message);
         dbus_connection_send(connection, msg_ret, NULL);
         dbus_message_unref(msg_ret);
         
@@ -387,6 +403,58 @@ xfmedia_dbus_message(DBusConnection *connection, DBusMessage *message,
         
         xfmedia_quit(mwin, XFMEDIA_QUIT_MODE_EXT);
         
+        ret = DBUS_HANDLER_RESULT_HANDLED;
+    } else if(dbus_message_is_method_call(message, XFMEDIA_DBUS_INTERFACE,
+            XFMEDIA_REMOTE_NOW_PLAYING))
+    {
+        gint status, now_playing;
+        gchar *title = NULL, *filename = NULL;
+        gint length = -1;
+
+        status = xfmedia_xine_get_status(mwin->xfx);
+        if(status == XINE_STATUS_PLAY
+           && (now_playing = xfmedia_playlist_entry_ref_get_index(mwin->cur_playing)) != -1
+           && xfmedia_playlist_get(mwin->plist, now_playing, &title, &length, &filename)) 
+        {
+            if(!title)
+                title = g_strdup(_("(no title)"));
+            
+            msg_ret = dbus_message_new_method_return(message);
+            dbus_message_append_args(msg_ret,
+#ifdef DBUS_USE_OLD_API
+                    DBUS_TYPE_INT32, now_playing,
+                    DBUS_TYPE_STRING, title,
+                    DBUS_TYPE_INT32, length,
+                    DBUS_TYPE_STRING, filename,
+#else
+                    DBUS_TYPE_INT32, &now_playing,
+                    DBUS_TYPE_STRING, &title,
+                    DBUS_TYPE_INT32, &length,
+                    DBUS_TYPE_STRING, &filename,
+#endif
+                    DBUS_TYPE_INVALID);
+            
+            g_free(title);
+            g_free(filename);
+        } else {
+            msg_ret = dbus_message_new_error(message, XFMEDIA_DBUS_ERROR,
+                    XFMEDIA_REMOTE_NOW_PLAYING);
+        }
+        dbus_connection_send(connection, msg_ret, NULL);
+        dbus_message_unref(msg_ret);
+         
+        ret = DBUS_HANDLER_RESULT_HANDLED;
+    } else if(dbus_message_is_method_call(message, XFMEDIA_DBUS_INTERFACE,
+            XFMEDIA_REMOTE_TRIGGER_JTF))
+    {
+        xfmedia_mainwin_show(mwin);
+        gtk_widget_grab_focus(mwin->window);
+        xfmedia_playlist_trigger_jump_to_file(mwin->plist);
+        
+        msg_ret = dbus_message_new_method_return(message);
+        dbus_connection_send(connection, msg_ret, NULL);
+        dbus_message_unref(msg_ret);
+        
         ret = DBUS_HANDLER_RESULT_HANDLED;
     }
     
@@ -394,7 +462,7 @@ xfmedia_dbus_message(DBusConnection *connection, DBusMessage *message,
 }
 
 gint
-xfmedia_remote_init(XfMediaMainwin *mwin)
+xfmedia_remote_init(XfmediaMainwin *mwin)
 {
     DBusConnection *dbus_conn;
     GError *err = NULL;
@@ -407,7 +475,7 @@ xfmedia_remote_init(XfMediaMainwin *mwin)
     
     dbus_g_thread_init();
     
-    dbus_conn = xfmedia_dbus_get_bus_connection();
+    dbus_conn = __xfmedia_dbus_get_bus_connection();
     if(G_UNLIKELY(!dbus_conn)) {
         if(err) {
             g_warning("Failed to connect to D-BUS session instance (%d): %s." \
@@ -457,13 +525,13 @@ xfmedia_remote_init(XfMediaMainwin *mwin)
 }
 
 void
-xfmedia_remote_cleanup(XfMediaMainwin *mwin)
+xfmedia_remote_cleanup(XfmediaMainwin *mwin)
 {
     DBusConnection *dbus_conn;
     
     g_return_if_fail(mwin);
     
-    dbus_conn = xfmedia_dbus_get_bus_connection();
+    dbus_conn = __xfmedia_dbus_get_bus_connection();
     
     if(G_UNLIKELY(!dbus_conn || !dbus_connection_get_is_connected(dbus_conn)))
         return;
@@ -476,292 +544,10 @@ xfmedia_remote_cleanup(XfMediaMainwin *mwin)
     }
 }
 
-
-/*******************************
- * client interface
- *******************************/
-
-static XfMediaRemoteStatus
-remote_do_ping(DBusConnection *dbus_conn, gint session_id)
-{
-    XfMediaRemoteStatus ret = XFMEDIA_REMOTE_COMMAND_FAILED;
-    DBusMessage *msg, *msg_ret;
-    DBusError derr;
-    gchar service_name[64], path_name[64];
-    
-    g_snprintf(service_name, 64, XFMEDIA_DBUS_SERVICE_FMT, session_id);
-    g_snprintf(path_name, 64, XFMEDIA_DBUS_PATH_FMT, session_id);
-    msg = dbus_message_new_method_call(service_name, path_name,
-            XFMEDIA_DBUS_INTERFACE, XFMEDIA_REMOTE_IS_RUNNING);
-    dbus_message_set_auto_activation(msg, FALSE);
-    
-    dbus_error_init(&derr);
-    msg_ret = dbus_connection_send_with_reply_and_block(dbus_conn,
-            msg, 5000, &derr);
-    dbus_message_unref(msg);
-    
-    if(!msg_ret) {
-        DBG("Unable to contact Xfmedia instance on id %d", session_id);
-        dbus_error_free(&derr);
-    } else if(!dbus_message_is_error(msg_ret, XFMEDIA_DBUS_ERROR))
-        ret = session_id;
-    
-    if(msg_ret)
-        dbus_message_unref(msg_ret);
-    
-    return ret;
-}
-
-XfMediaRemoteStatus
-#ifdef HAVE_STDARG_H
-xfmedia_remote_send_command(gint session_id, const gchar *command, ...)
-#else /* ifdef HAVE_VARARGS_H */
-xfmedia_remote_send_command(va_alist) va_dcl
-#endif
-{
-    XfMediaRemoteStatus ret = XFMEDIA_REMOTE_UNKNOWN_ERROR;
-    va_list ap;
-    DBusConnection *dbus_conn;
-    DBusMessage *msg, *msg_ret;
-    DBusError derr;
-    gchar service_name[64], path_name[64];
-    
-    dbus_conn = xfmedia_dbus_get_bus_connection();
-    if(!dbus_conn)
-        return XFMEDIA_REMOTE_NO_SESSION_BUS;
-    
-#ifdef HAVE_STDARG_H
-    va_start(ap, command);
-#else /* ifdef HAVE_VARARGS_H */
-    gint session_id;
-    const gchar *command;
-    
-    va_start(ap);
-    session_id = va_arg(ap, gint);
-    command = va_arg(ap, const gchar *);
-#endif
-    
-    g_snprintf(service_name, 64, XFMEDIA_DBUS_SERVICE_FMT, session_id);
-    g_snprintf(path_name, 64, XFMEDIA_DBUS_PATH_FMT, session_id);
-    
-    if(!strcmp(command, XFMEDIA_REMOTE_IS_RUNNING)) {
-        if(session_id >= 0)
-            ret = remote_do_ping(dbus_conn, session_id);
-        else {
-            gint i;
-            for(i = 0; i < MAX_INSTANCES; i++) {
-                ret = remote_do_ping(dbus_conn, i);
-                if(ret != XFMEDIA_REMOTE_COMMAND_FAILED)
-                    break;
-            }
-        }
-    } else if(!strcmp(command, XFMEDIA_REMOTE_CLEAR_PLAYLIST)) {
-        msg = dbus_message_new_method_call(service_name, path_name,
-                XFMEDIA_DBUS_INTERFACE, XFMEDIA_REMOTE_CLEAR_PLAYLIST);
-        dbus_message_set_auto_activation(msg, FALSE);
-        
-        dbus_error_init(&derr);
-        msg_ret = dbus_connection_send_with_reply_and_block(dbus_conn,
-                msg, 5000, &derr);
-        dbus_message_unref(msg);
-        
-        if(!msg_ret) {
-            ret = XFMEDIA_REMOTE_UNKNOWN_ERROR;
-            dbus_error_free(&derr);
-        } else if(dbus_message_is_error(msg_ret, XFMEDIA_DBUS_ERROR))
-            ret = XFMEDIA_REMOTE_COMMAND_FAILED;
-        else if(msg_ret)
-            ret = XFMEDIA_REMOTE_COMMAND_SUCCEEDED;
-        
-        if(msg_ret)
-            dbus_message_unref(msg_ret);
-    } else if(!strcmp(command, XFMEDIA_REMOTE_LOAD_PLAYLIST)
-            || !strcmp(command, XFMEDIA_REMOTE_SAVE_PLAYLIST))
-    {
-        const gchar *filename;
-        
-        filename = va_arg(ap, const gchar *);
-        if(!filename || !*filename)
-            ret = XFMEDIA_REMOTE_BAD_ARGUMENTS;
-        else {
-            msg = dbus_message_new_method_call(service_name, path_name,
-                    XFMEDIA_DBUS_INTERFACE, command);
-            dbus_message_set_auto_activation(msg, FALSE);
-            dbus_message_append_args(msg,
-#ifdef DBUS_USE_OLD_API
-                    DBUS_TYPE_STRING, filename,
-#else
-                    DBUS_TYPE_STRING, &filename,
-#endif
-                    DBUS_TYPE_INVALID);
-            
-            dbus_error_init(&derr);
-            msg_ret = dbus_connection_send_with_reply_and_block(dbus_conn,
-                    msg, 5000, &derr);
-            dbus_message_unref(msg);
-            
-            if(!msg_ret) {
-                ret = XFMEDIA_REMOTE_UNKNOWN_ERROR;
-                dbus_error_free(&derr);
-            } else if(dbus_message_is_error(msg_ret, XFMEDIA_DBUS_ERROR))
-                ret = XFMEDIA_REMOTE_COMMAND_FAILED;
-            else if(msg_ret)
-                ret = XFMEDIA_REMOTE_COMMAND_SUCCEEDED;
-            
-            if(msg_ret)
-                dbus_message_unref(msg_ret);
-        }
-    } else if(!strcmp(command, XFMEDIA_REMOTE_ADD_FILE)) {
-        const gchar *filename;
-        gint index;
-        
-        filename = va_arg(ap, const gchar *);
-        index = va_arg(ap, gint);
-        if(index < -1)
-            index = -1;
-        
-        if(!filename || !*filename)
-            ret = XFMEDIA_REMOTE_BAD_ARGUMENTS;
-        else {
-            msg = dbus_message_new_method_call(service_name, path_name,
-                    XFMEDIA_DBUS_INTERFACE, XFMEDIA_REMOTE_ADD_FILE);
-            dbus_message_set_auto_activation(msg, FALSE);
-            dbus_message_append_args(msg,
-#ifdef DBUS_USE_OLD_API
-                    DBUS_TYPE_STRING, filename,
-                    DBUS_TYPE_INT32, index,
-#else
-                    DBUS_TYPE_STRING, &filename,
-                    DBUS_TYPE_INT32, &index,
-#endif
-                    DBUS_TYPE_INVALID);
-            
-            dbus_error_init(&derr);
-            msg_ret = dbus_connection_send_with_reply_and_block(dbus_conn,
-                    msg, 5000, &derr);
-            dbus_message_unref(msg);
-            
-            if(!msg_ret) {
-                ret = XFMEDIA_REMOTE_UNKNOWN_ERROR;
-                dbus_error_free(&derr);
-            } else if(dbus_message_is_error(msg_ret, XFMEDIA_DBUS_ERROR))
-                ret = XFMEDIA_REMOTE_COMMAND_FAILED;
-            else if(msg_ret) {
-                dbus_error_init(&derr);
-                if(!dbus_message_get_args(msg_ret, &derr,
-                        DBUS_TYPE_INT32, &ret, DBUS_TYPE_INVALID))
-                {
-                    ret = XFMEDIA_REMOTE_COMMAND_FAILED;
-                    dbus_error_free(&derr);
-                } else
-                    ret = XFMEDIA_REMOTE_COMMAND_SUCCEEDED;
-            }
-            
-            if(msg_ret)
-                dbus_message_unref(msg_ret);
-        }
-    } else if(!strcmp(command, XFMEDIA_REMOTE_REMOVE_FILE)) {
-        guint index;
-        
-        index = va_arg(ap, guint);
-        
-        msg = dbus_message_new_method_call(service_name, path_name,
-                XFMEDIA_DBUS_INTERFACE, XFMEDIA_REMOTE_REMOVE_FILE);
-        dbus_message_set_auto_activation(msg, FALSE);
-        dbus_message_append_args(msg,
-#ifdef DBUS_USE_OLD_API
-                DBUS_TYPE_INT32, index,
-#else
-                DBUS_TYPE_INT32, &index,
-#endif
-                DBUS_TYPE_INVALID);
-        
-        dbus_error_init(&derr);
-        msg_ret = dbus_connection_send_with_reply_and_block(dbus_conn,
-                msg, 5000, &derr);
-        dbus_message_unref(msg);
-        
-        if(!msg_ret) {
-            ret = XFMEDIA_REMOTE_UNKNOWN_ERROR;
-            dbus_error_free(&derr);
-        } else if(dbus_message_is_error(msg_ret, XFMEDIA_DBUS_ERROR))
-            ret = XFMEDIA_REMOTE_COMMAND_FAILED;
-        else if(msg_ret)
-            ret = XFMEDIA_REMOTE_COMMAND_SUCCEEDED;
-        
-        if(msg_ret)
-            dbus_message_unref(msg_ret);
-    } else if(!strcmp(command, XFMEDIA_REMOTE_PLAY)) {
-        gint index;
-        
-        index = va_arg(ap, gint);
-        if(index < -1)
-            index = -1;
-        
-        msg = dbus_message_new_method_call(service_name, path_name,
-                XFMEDIA_DBUS_INTERFACE, XFMEDIA_REMOTE_PLAY);
-        dbus_message_set_auto_activation(msg, FALSE);
-        dbus_message_append_args(msg,
-#ifdef DBUS_USE_OLD_API
-                DBUS_TYPE_INT32, index,
-#else
-                DBUS_TYPE_INT32, &index,
-#endif
-                DBUS_TYPE_INVALID);
-        
-        dbus_error_init(&derr);
-        msg_ret = dbus_connection_send_with_reply_and_block(dbus_conn,
-                msg, 5000, &derr);
-        dbus_message_unref(msg);
-        
-        if(!msg_ret) {
-            ret = XFMEDIA_REMOTE_UNKNOWN_ERROR;
-            dbus_error_free(&derr);
-        } else if(dbus_message_is_error(msg_ret, XFMEDIA_DBUS_ERROR))
-            ret = XFMEDIA_REMOTE_COMMAND_FAILED;
-        else if(msg_ret)
-            ret = XFMEDIA_REMOTE_COMMAND_SUCCEEDED;
-        
-        if(msg_ret)
-            dbus_message_unref(msg_ret);
-    } else if(!strcmp(command, XFMEDIA_REMOTE_PAUSE)
-            || !strcmp(command, XFMEDIA_REMOTE_STOP)
-            || !strcmp(command, XFMEDIA_REMOTE_PREV)
-            || !strcmp(command, XFMEDIA_REMOTE_NEXT)
-            || !strcmp(command, XFMEDIA_REMOTE_QUIT))
-    {
-        msg = dbus_message_new_method_call(service_name, path_name,
-                XFMEDIA_DBUS_INTERFACE, command);
-        dbus_message_set_auto_activation(msg, FALSE);
-        
-        dbus_error_init(&derr);
-        msg_ret = dbus_connection_send_with_reply_and_block(dbus_conn,
-                msg, 5000, &derr);
-        dbus_message_unref(msg);
-        
-        if(!msg_ret) {
-            ret = XFMEDIA_REMOTE_UNKNOWN_ERROR;
-            dbus_error_free(&derr);
-        } else if(dbus_message_is_error(msg_ret, XFMEDIA_DBUS_ERROR))
-            ret = XFMEDIA_REMOTE_COMMAND_FAILED;
-        else if(msg_ret)
-            ret = XFMEDIA_REMOTE_COMMAND_SUCCEEDED;
-        
-        if(msg_ret)
-            dbus_message_unref(msg_ret);
-    } else {
-        ret = XFMEDIA_REMOTE_BAD_COMMAND;
-    }
-    
-    va_end(ap);
-    
-    return ret;
-}
-
 #else /* !defined(HAVE_DBUS) */
 
 gint
-xfmedia_remote_init(XfMediaMainwin *mwin)
+xfmedia_remote_init(XfmediaMainwin *mwin)
 {
     g_warning("Not compiled with D-BUS support.  Remote control interface is not available");
     
@@ -769,16 +555,9 @@ xfmedia_remote_init(XfMediaMainwin *mwin)
 }
 
 void
-xfmedia_remote_cleanup(XfMediaMainwin *mwin)
+xfmedia_remote_cleanup(XfmediaMainwin *mwin)
 {
     /* noop */
 }
 
-XfMediaRemoteStatus
-xfmedia_remote_send_command(gint session_id, const gchar *command, ...)
-{
-    g_critical(_("Xfmedia was not compiled with D-BUS support.  Remote control interface is not available."));
-    return XFMEDIA_REMOTE_UNKNOWN_ERROR;
-}
-
 #endif /* !defined(HAVE_DBUS) */
