@@ -56,7 +56,7 @@
 
 struct PConfData
 {
-    GtkWidget *gtx;
+    GtkWidget *xfx;
     GtkWidget *cfgwin;
     GtkTreeSelection *treesel;
     const gchar *title;
@@ -79,10 +79,263 @@ enum
 
 static gboolean sd_running = FALSE;
 
+static gboolean
+xfmedia_xfmedia_xine_cfgedit_str_cb(GtkWidget *w, GdkEventFocus *evt,
+        gpointer user_data)
+{
+    XfmediaMainwin *mwin = user_data;
+    const gchar *key = g_object_get_data(G_OBJECT(w), "xfmedia-xinecfg-key");
+    gchar *value;
+    xine_cfg_entry_t cfg;
+    
+    g_return_val_if_fail(key && *key, FALSE);
+    
+    if(!xfmedia_xine_config_lookup_entry(mwin->xfx, key, &cfg))
+        return FALSE;
+    
+    value = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
+    if(value) {
+        if(strcmp(value, cfg.str_value)) {
+            cfg.str_value = value;
+            xfmedia_xine_config_update_entry(mwin->xfx, &cfg);
+        }
+        g_free(value);
+    }
+    
+    return FALSE;
+}
+
+static void
+xfmedia_xfmedia_xine_cfgedit_enum_cb(GtkComboBox *combo, gpointer user_data)
+{
+    XfmediaMainwin *mwin = user_data;
+    const gchar *key = g_object_get_data(G_OBJECT(combo), "xfmedia-xinecfg-key");
+    gint value;
+    xine_cfg_entry_t cfg;
+    
+    g_return_if_fail(key && *key);
+    
+    if(!xfmedia_xine_config_lookup_entry(mwin->xfx, key, &cfg))
+        return;
+    
+    value = gtk_combo_box_get_active(combo);
+    if(value != cfg.num_value) {
+        cfg.num_value = value;
+        xfmedia_xine_config_update_entry(mwin->xfx, &cfg);
+        
+        if(!strcmp(key, "audio.driver"))
+            xfmedia_xine_set_audio_output_plugin(mwin->xfx, cfg.enum_values[value]);
+        else if(!strcmp(key, "video.driver"))
+            xfmedia_xine_set_video_output_plugin(mwin->xfx, cfg.enum_values[value]);
+    }
+}
+
+static GtkWidget *
+xfmedia_settings_create_devices_tab(XfmediaMainwin *mwin, GtkWidget *parent)
+{
+    GtkWidget *topvbox, *frame, *frame_bin, *vbox, *hbox, *lbl, *entry, *combo;
+    GtkSizeGroup *sg;
+    xine_cfg_entry_t cfg;
+    GList *plugins, *l;
+    
+    topvbox = gtk_vbox_new(FALSE, BORDER/2);
+    gtk_container_set_border_width(GTK_CONTAINER(topvbox), BORDER/2);
+    gtk_widget_show(topvbox);
+    
+    frame = xfmedia_create_framebox(_("Optical Devices"), &frame_bin);
+    gtk_widget_show(frame);
+    gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
+    
+    vbox = gtk_vbox_new(FALSE, BORDER/2);
+    gtk_widget_show(vbox);
+    gtk_container_add(GTK_CONTAINER(frame_bin), vbox);
+    
+    sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+    
+    hbox = gtk_hbox_new(FALSE, BORDER/2);
+    gtk_widget_show(hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    
+    lbl = gtk_label_new_with_mnemonic(_("_CD Audio Device:"));
+    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
+    gtk_size_group_add_widget(sg, lbl);
+    gtk_widget_show(lbl);
+    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
+    
+    entry = gtk_entry_new();
+    if(xfmedia_xine_config_lookup_entry(mwin->xfx, "media.audio_cd.device", &cfg)) {
+        gtk_entry_set_text(GTK_ENTRY(entry), cfg.str_value);
+        g_object_set_data(G_OBJECT(entry), "xfmedia-xinecfg-key",
+                "media.audio_cd.device");
+        g_signal_connect(G_OBJECT(entry), "focus-out-event",
+                G_CALLBACK(xfmedia_xfmedia_xine_cfgedit_str_cb), mwin);
+    }
+    gtk_widget_show(entry);
+    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), entry);
+    
+    hbox = gtk_hbox_new(FALSE, BORDER/2);
+    gtk_widget_show(hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    
+    lbl = gtk_label_new_with_mnemonic(_("_DVD Video Device:"));
+    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
+    gtk_size_group_add_widget(sg, lbl);
+    gtk_widget_show(lbl);
+    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
+    
+    entry = gtk_entry_new();
+    if(xfmedia_xine_config_lookup_entry(mwin->xfx, "media.dvd.device", &cfg)) {
+        gtk_entry_set_text(GTK_ENTRY(entry), cfg.str_value);
+        g_object_set_data(G_OBJECT(entry), "xfmedia-xinecfg-key",
+                "media.dvd.device");
+        g_signal_connect(G_OBJECT(entry), "focus-out-event",
+                G_CALLBACK(xfmedia_xfmedia_xine_cfgedit_str_cb), mwin);
+    }
+    gtk_widget_show(entry);
+    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), entry);
+    
+    hbox = gtk_hbox_new(FALSE, BORDER/2);
+    gtk_widget_show(hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    
+    lbl = gtk_label_new_with_mnemonic(_("_Video CD Device:"));
+    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
+    gtk_size_group_add_widget(sg, lbl);
+    gtk_widget_show(lbl);
+    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
+    
+    entry = gtk_entry_new();
+    if(xfmedia_xine_config_lookup_entry(mwin->xfx, "media.vcd.device", &cfg)) {
+        gtk_entry_set_text(GTK_ENTRY(entry), cfg.str_value);
+        g_object_set_data(G_OBJECT(entry), "xfmedia-xinecfg-key",
+                "media.vcd.device");
+        g_signal_connect(G_OBJECT(entry), "focus-out-event",
+                G_CALLBACK(xfmedia_xfmedia_xine_cfgedit_str_cb), mwin);
+    }
+    gtk_widget_show(entry);
+    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), entry);
+    
+    frame = xfmedia_create_framebox(_("Audio"), &frame_bin);
+    gtk_widget_show(frame);
+    gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
+    
+    vbox = gtk_vbox_new(FALSE, BORDER/2);
+    gtk_widget_show(vbox);
+    gtk_container_add(GTK_CONTAINER(frame_bin), vbox);
+    
+    hbox = gtk_hbox_new(FALSE, BORDER/2);
+    gtk_widget_show(hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    
+    lbl = gtk_label_new_with_mnemonic(_("_Driver:"));
+    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
+    gtk_size_group_add_widget(sg, lbl);
+    gtk_widget_show(lbl);
+    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
+    
+    combo = gtk_combo_box_new_text();
+    plugins = xfmedia_xine_list_audio_output_plugins(mwin->xfx);
+    xfmedia_xine_config_lookup_entry(mwin->xfx, "audio.driver", &cfg);
+    if(!plugins) {
+        gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("(no drivers)"));
+        gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
+    } else {
+        gint i, selected = -1;
+        for(l = plugins, i = 0; l; l = l->next, i++) {
+            gtk_combo_box_append_text(GTK_COMBO_BOX(combo), l->data);
+            if(!strcmp(l->data, cfg.enum_values[cfg.num_value]))
+                selected = i;
+        }
+        if(selected != -1)
+            gtk_combo_box_set_active(GTK_COMBO_BOX(combo), selected);
+        g_object_set_data(G_OBJECT(combo), "xfmedia-xinecfg-key",
+                "audio.driver");
+        g_signal_connect(G_OBJECT(combo), "changed",
+                G_CALLBACK(xfmedia_xfmedia_xine_cfgedit_enum_cb), mwin);
+    }
+    gtk_widget_show(combo);
+    gtk_box_pack_start(GTK_BOX(hbox), combo, TRUE, TRUE, 0);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), combo);
+    
+    hbox = gtk_hbox_new(FALSE, BORDER/2);
+    gtk_widget_show(hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    
+    lbl = gtk_label_new_with_mnemonic(_("_Speaker Layout:"));
+    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
+    gtk_size_group_add_widget(sg, lbl);
+    gtk_widget_show(lbl);
+    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
+    
+    combo = gtk_combo_box_new_text();
+    if(!xfmedia_xine_config_lookup_entry(mwin->xfx, "audio.output.speaker_arrangement", &cfg))
+        gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("(option unavailable)"));
+    else {
+        gint i;
+        for(i = 0; cfg.enum_values[i]; i++)
+            gtk_combo_box_append_text(GTK_COMBO_BOX(combo), cfg.enum_values[i]);
+        gtk_combo_box_set_active(GTK_COMBO_BOX(combo), cfg.num_value);
+        g_object_set_data(G_OBJECT(combo), "xfmedia-xinecfg-key",
+                "audio.output.speaker_arrangement");
+        g_signal_connect(G_OBJECT(combo), "changed",
+                G_CALLBACK(xfmedia_xfmedia_xine_cfgedit_enum_cb), mwin);
+    }
+    gtk_widget_show(combo);
+    gtk_box_pack_start(GTK_BOX(hbox), combo, TRUE, TRUE, 0);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), combo);
+    
+    frame = xfmedia_create_framebox(_("Video"), &frame_bin);
+    gtk_widget_show(frame);
+    gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
+    
+    vbox = gtk_vbox_new(FALSE, BORDER/2);
+    gtk_widget_show(vbox);
+    gtk_container_add(GTK_CONTAINER(frame_bin), vbox);
+    
+    hbox = gtk_hbox_new(FALSE, BORDER/2);
+    gtk_widget_show(hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    
+    lbl = gtk_label_new_with_mnemonic(_("D_river:"));
+    gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
+    gtk_size_group_add_widget(sg, lbl);
+    gtk_widget_show(lbl);
+    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
+    
+    combo = gtk_combo_box_new_text();
+    plugins = xfmedia_xine_list_video_output_plugins(mwin->xfx);
+    xfmedia_xine_config_lookup_entry(mwin->xfx, "video.driver", &cfg);
+    if(!plugins) {
+        gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("(no drivers)"));
+        gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
+    } else {
+        gint i, selected = -1;
+        for(l = plugins, i = 0; l; l = l->next, i++) {
+            gtk_combo_box_append_text(GTK_COMBO_BOX(combo), l->data);
+            if(!strcmp(l->data, cfg.enum_values[cfg.num_value]))
+                selected = i;
+        }
+        if(selected != -1)
+            gtk_combo_box_set_active(GTK_COMBO_BOX(combo), selected);
+        g_object_set_data(G_OBJECT(combo), "xfmedia-xinecfg-key",
+                "video.driver");
+        g_signal_connect(G_OBJECT(combo), "changed",
+                G_CALLBACK(xfmedia_xfmedia_xine_cfgedit_enum_cb), mwin);
+    }
+    gtk_widget_show(combo);
+    gtk_box_pack_start(GTK_BOX(hbox), combo, TRUE, TRUE, 0);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), combo);
+    
+    return topvbox;
+}
+
 static gboolean
 titledisp_cb(GtkWidget *w, GdkEventFocus *evt, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     gchar *fmt;
     gint i, tot;
     
@@ -108,7 +361,7 @@ autoadvance_cb(GtkToggleButton *tb, gpointer user_data)
 static void
 allws_cb(GtkToggleButton *tb, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     gboolean set;
     
     set = gtk_toggle_button_get_active(tb);
@@ -123,7 +376,7 @@ allws_cb(GtkToggleButton *tb, gpointer user_data)
 static void
 alwaysontop_cb(GtkToggleButton *tb, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     gboolean set;
     
     set = gtk_toggle_button_get_active(tb);
@@ -134,11 +387,14 @@ alwaysontop_cb(GtkToggleButton *tb, gpointer user_data)
 static void
 trayicon_cb(GtkToggleButton *tb, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     gboolean set;
     
     set = gtk_toggle_button_get_active(tb);
+    
     gtk_widget_set_sensitive(mwin->tray_mini_chk, set);
+    gtk_widget_set_sensitive(mwin->tray_middle_box, set);
+    
     if(set) {
         xfmedia_mainwin_activate_systray(mwin);
         if(mwin->is_iconified
@@ -157,7 +413,7 @@ trayicon_cb(GtkToggleButton *tb, gpointer user_data)
 static void
 trayiconmini_cb(GtkToggleButton *tb, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     gboolean set;
     
     set = gtk_toggle_button_get_active(tb);
@@ -170,6 +426,13 @@ trayiconmini_cb(GtkToggleButton *tb, gpointer user_data)
     xfmedia_settings_set_bool("/xfmedia/general/minimize_to_tray", set);
 }
 
+static void
+tray_middle_combo_changed_cb(GtkComboBox *combo, gpointer user_data)
+{
+    xfmedia_settings_set_int("/xfmedia/general/tray_middle_click_action",
+            gtk_combo_box_get_active(combo));
+}
+
 static void
 sd_destroy(struct SDData *sddata)
 {
@@ -192,8 +455,11 @@ soea_toggle_cb(GtkToggleButton *tb, gpointer user_data)
 static void
 dock_autohide_cb(GtkToggleButton *tb, gpointer user_data)
 {
-    xfmedia_settings_set_bool("/xfmedia/general/dock_autohide",
-            gtk_toggle_button_get_active(tb));
+    XfmediaMainwin *mwin = user_data;
+    gboolean set = gtk_toggle_button_get_active(tb);
+    
+    xfmedia_settings_set_bool("/xfmedia/general/dock_autohide", set);
+    xfmedia_mainwin_set_autohide(mwin, set);
 }
 
 static void
@@ -208,17 +474,17 @@ metadata_load_change_cb(GtkToggleButton *tb, gpointer user_data)
 static void
 audio_vis_changed_cb(GtkMenuItem *mi, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     GtkLabel *lbl = GTK_LABEL(gtk_bin_get_child(GTK_BIN(mi)));
     const gchar *vis_name = gtk_label_get_text(lbl);
     
     xfmedia_settings_set_string("/xfmedia/general/selected_vis", vis_name);
     
     if(xfmedia_settings_get_bool("/xfmedia/general/show_audio_vis")
-            && gtk_xine_get_status(mwin->gtx) == XINE_STATUS_PLAY
-            && !gtk_xine_get_stream_info(mwin->gtx, XINE_STREAM_INFO_HAS_VIDEO))
+            && xfmedia_xine_get_status(mwin->xfx) == XINE_STATUS_PLAY
+            && !xfmedia_xine_get_stream_info(mwin->xfx, XINE_STREAM_INFO_HAS_VIDEO))
     {
-        gtk_xine_set_vis(mwin->gtx, vis_name);
+        xfmedia_xine_set_vis(mwin->xfx, vis_name);
     }
 }
 
@@ -227,7 +493,7 @@ audio_vis_changed_cb(GtkMenuItem *mi, gpointer user_data)
 static gint
 plugins_sort_alpha(gconstpointer a, gconstpointer b)
 {
-    XfMediaPlugin *pa = (XfMediaPlugin *)a, *pb = (XfMediaPlugin *)b;
+    XfmediaPlugin *pa = (XfmediaPlugin *)a, *pb = (XfmediaPlugin *)b;
     
     return g_ascii_strcasecmp(xfmedia_plugin_get_name(pa), xfmedia_plugin_get_name(pb));
 }
@@ -242,7 +508,7 @@ plugin_enabled_toggle_cb(GtkCellRendererToggle *crt, gchar *arg1,
 }
 
 static GtkWidget *
-add_plugins_tab(XfMediaMainwin *mwin)
+add_plugins_tab(XfmediaMainwin *mwin)
 {
     GtkWidget *topvbox, *hbox, *sw, *treeview, *btn;
     GtkListStore *ls;
@@ -268,7 +534,7 @@ add_plugins_tab(XfMediaMainwin *mwin)
     if(plugins)
         g_list_sort(plugins, plugins_sort_alpha);
     for(l = plugins; l; l = l->next) {
-        XfMediaPlugin *plugin = l->data;
+        XfmediaPlugin *plugin = l->data;
         
         gtk_list_store_append(ls, &itr);
         gtk_list_store_set(ls, &itr,
@@ -316,11 +582,11 @@ add_plugins_tab(XfMediaMainwin *mwin)
 #endif
 
 void
-xfmedia_settings_show_dialog(XfMediaMainwin *mwin)
+xfmedia_settings_show_dialog(XfmediaMainwin *mwin)
 {
     static GtkWidget *win = NULL;
-    GtkWidget *topvbox, *vbox, *hbox, *btn, *frame, *entry, *lbl;
-    GtkWidget *notebook, *menu, *mi, *cb;
+    GtkWidget *topvbox, *vbox, *hbox, *btn, *frame, *frame_bin, *entry, *lbl,
+              *combo, *notebook, *menu, *mi, *cb;
     GtkTooltips *ttips;
     struct SDData *sddata;
     GList *vis_plugins;
@@ -361,14 +627,14 @@ xfmedia_settings_show_dialog(XfMediaMainwin *mwin)
     gtk_container_set_border_width(GTK_CONTAINER(topvbox), BORDER/2);
     gtk_widget_show(topvbox);
     
-    frame = xfce_framebox_new(_("Playlist"), TRUE);
+    frame = xfmedia_create_framebox(_("Playlist"), &frame_bin);
     gtk_widget_show(frame);
     gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
     
     vbox = gtk_vbox_new(FALSE, BORDER/2);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER/2);
     gtk_widget_show(vbox);
-    xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox);
+    gtk_container_add(GTK_CONTAINER(frame_bin), vbox);
     
     hbox = gtk_hbox_new(FALSE, BORDER);
     gtk_widget_show(hbox);
@@ -437,7 +703,7 @@ xfmedia_settings_show_dialog(XfMediaMainwin *mwin)
             _("Load file metadata for entries in the playlist when they are visible."), NULL);
     
     btn = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(btn),
-            _("On _play"));
+            _("On pla_y"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
             (xfmedia_settings_get_int("/xfmedia/playlist/metadata_load_impetus") == 1));
     gtk_widget_show(btn);
@@ -458,14 +724,14 @@ xfmedia_settings_show_dialog(XfMediaMainwin *mwin)
     gtk_tooltips_set_tip(ttips, btn,
             _("Never load file metadata for entries in the playlist."), NULL);
     
-    frame = xfce_framebox_new(_("Window"), TRUE);
+    frame = xfmedia_create_framebox(_("Window"), &frame_bin);
     gtk_widget_show(frame);
     gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
     
     vbox = gtk_vbox_new(FALSE, BORDER/2);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER/2);
     gtk_widget_show(vbox);
-    xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox);
+    gtk_container_add(GTK_CONTAINER(frame_bin), vbox);
     
     btn = gtk_check_button_new_with_mnemonic(_("Show Xfmedia on all _workspaces"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
@@ -493,14 +759,14 @@ xfmedia_settings_show_dialog(XfMediaMainwin *mwin)
     gtk_tooltips_set_tip(ttips, btn,
             _("When the Xfmedia window is dragged to a screen edge, it will auto-hide itself and reappear when you move the mouse to the edge of the screen."), NULL);
     
-    frame = xfce_framebox_new(_("Tray Icon"), TRUE);
+    frame = xfmedia_create_framebox(_("Tray Icon"), &frame_bin);
     gtk_widget_show(frame);
     gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
     
     vbox = gtk_vbox_new(FALSE, BORDER/2);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER/2);
     gtk_widget_show(vbox);
-    xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox);
+    gtk_container_add(GTK_CONTAINER(frame_bin), vbox);
     
     btn = gtk_check_button_new_with_mnemonic(_("Show _tray icon"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
@@ -520,6 +786,32 @@ xfmedia_settings_show_dialog(XfMediaMainwin *mwin)
     g_signal_connect(G_OBJECT(btn), "toggled", G_CALLBACK(trayiconmini_cb), mwin);
     gtk_tooltips_set_tip(ttips, btn, _("Minimizes Xfmedia to the tray icon rather than to the taskbar or iconbox."), NULL);
     
+    mwin->tray_middle_box = hbox = gtk_hbox_new(FALSE, BORDER/2);
+    gtk_widget_show(hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    
+    lbl = gtk_label_new_with_mnemonic(_("On middle _click:"));
+    gtk_widget_show(lbl);
+    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
+    gtk_tooltips_set_tip(ttips, lbl, _("The tray icon can perform an action when middle-clicked.  Select the action in the dropdown box."), NULL);
+    
+    combo = gtk_combo_box_new_text();
+    gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("Do nothing"));
+    gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("Toggle play/pause"));
+    gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("Next track"));
+    gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("Previous track"));
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combo),
+            xfmedia_settings_get_int("/xfmedia/general/tray_middle_click_action"));
+    gtk_widget_show(combo);
+    gtk_box_pack_start(GTK_BOX(hbox), combo, FALSE, FALSE, 0);
+    g_signal_connect(G_OBJECT(combo), "changed",
+            G_CALLBACK(tray_middle_combo_changed_cb), mwin);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), combo);
+    gtk_tooltips_set_tip(ttips, combo, _("The tray icon can perform an action when middle-clicked.  Select the action in the dropdown box."), NULL);
+    
+    gtk_widget_set_sensitive(hbox,
+            xfmedia_settings_get_bool("/xfmedia/general/show_tray_icon"));
+    
     lbl = gtk_label_new_with_mnemonic(_("O_ptions"));
     gtk_widget_show(lbl);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), topvbox, lbl);
@@ -531,18 +823,18 @@ xfmedia_settings_show_dialog(XfMediaMainwin *mwin)
     gtk_container_set_border_width(GTK_CONTAINER(topvbox), BORDER/2);
     gtk_widget_show(topvbox);
     
-    frame = xfce_framebox_new(_("Selected Visualization"), TRUE);
+    frame = xfmedia_create_framebox(_("Selected Visualization"), &frame_bin);
     gtk_widget_show(frame);
     gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
     
     hbox = gtk_hbox_new(FALSE, BORDER/2);
     gtk_widget_show(hbox);
-    xfce_framebox_add(XFCE_FRAMEBOX(frame), hbox);
+    gtk_container_add(GTK_CONTAINER(frame_bin), hbox);
     
     menu = gtk_menu_new();
     gtk_widget_show(menu);
     
-    vis_plugins = gtk_xine_list_post_plugins_typed(mwin->gtx,
+    vis_plugins = xfmedia_xine_list_post_plugins_typed(mwin->xfx,
             XINE_POST_TYPE_AUDIO_VISUALIZATION);
     if(vis_plugins) {
         GList *l;
@@ -570,14 +862,14 @@ xfmedia_settings_show_dialog(XfMediaMainwin *mwin)
     gtk_widget_show(cb);
     gtk_box_pack_start(GTK_BOX(hbox), cb, TRUE, TRUE, 0);
     
-    frame = xfce_framebox_new(_("Visualization Options"), TRUE);
+    frame = xfmedia_create_framebox(_("Visualization Options"), &frame_bin);
     gtk_widget_show(frame);
     gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
     
     vbox = gtk_vbox_new(FALSE, BORDER/2);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER/2);
     gtk_widget_show(vbox);
-    xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox);
+    gtk_container_add(GTK_CONTAINER(frame_bin), vbox);
     
     lbl = gtk_label_new("(not yet implemented)");
     gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
@@ -589,6 +881,12 @@ xfmedia_settings_show_dialog(XfMediaMainwin *mwin)
     gtk_widget_show(lbl);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), topvbox, lbl);
     
+    /* devices tab */
+    topvbox = xfmedia_settings_create_devices_tab(mwin, win);
+    lbl = gtk_label_new_with_mnemonic(_("_Devices"));
+    gtk_widget_show(lbl);
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), topvbox, lbl);
+    
     /* keybindings editor */
     topvbox = xfmedia_keybindings_create_editor(mwin, win);
     lbl = gtk_label_new_with_mnemonic(_("_Keybindings"));
