@@ -55,9 +55,11 @@
 #endif
 
 #include <libxfce4util/libxfce4util.h>
+#include <libxfcegui4/libxfcegui4.h>
 
 #include <xfmedia/xfmedia-settings.h>
 #include "xfmedia-common.h"
+#include "mainwin.h"
 #include "playlist-files.h"
 
 #define PLAYLISTFILE             "xfmedia/playlist.m3u"
@@ -65,7 +67,7 @@
 #define XFMEDIA_CONFIG_NEW       "xfmedia/settings.xml.new"
 #define XFMEDIA_CONFIG_WRITEFREQ 7500
 
-typedef struct _XfMediaConfigEntry
+typedef struct _XfmediaConfigEntry
 {
     gchar *key;
     gint type;
@@ -75,7 +77,7 @@ typedef struct _XfMediaConfigEntry
         gboolean b;
         gdouble f;
     } value;
-} XfMediaConfigEntry;
+} XfmediaConfigEntry;
 
 enum {
     XFMEDIA_SETTING_BRANCH = 0,
@@ -95,7 +97,7 @@ static gboolean settings_write_pending = FALSE;
 gint
 xfmedia_settings_glist_compare(gconstpointer a, gconstpointer b)
 {
-    const XfMediaConfigEntry *xa = a, *xb = b;
+    const XfmediaConfigEntry *xa = a, *xb = b;
     
     return strcmp(xa->key, xb->key);
 }
@@ -103,7 +105,7 @@ xfmedia_settings_glist_compare(gconstpointer a, gconstpointer b)
 static void
 ght_val_destroy(gpointer data)
 {
-    XfMediaConfigEntry *entry = data;
+    XfmediaConfigEntry *entry = data;
     
     g_free(entry->key);
     if(entry->type == XFMEDIA_SETTING_STRING && entry->value.s)
@@ -132,7 +134,7 @@ settings_write_xml_tree(FILE *fp, GList *prefs)
     gint depth = 1, new_depth;
     gchar tabs[128], *name_esc, *val_esc, *p;
     GList *l;
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     if(depth > 127)
         depth = 127;
@@ -236,21 +238,11 @@ settings_write(gpointer user_data)
     
     fclose(fp);
     
-    if(g_file_test(cfgfile, G_FILE_TEST_EXISTS)) {
-        if(!unlink(cfgfile)) {
-            if(!link(cfgfile_new, cfgfile)) {
-                if(unlink(cfgfile_new))
-                    g_warning("Unable to delete temp file when writing settings");
-            } else
-                g_warning("Unable to copy temp file to settings file");
-        } else
-            g_warning("Unable to remove old config file");
-    } else {
-        if(!link(cfgfile_new, cfgfile)) {
-            if(unlink(cfgfile_new))
-                g_warning("Unable to delete temp file when writing settings");
-        } else
-            g_warning("Unable to copy temp file to settings file");
+    if(rename(cfgfile_new, cfgfile)) {
+        xfce_message_dialog(NULL, _("Settings Save Error"),
+                GTK_STOCK_DIALOG_ERROR, _("The settings file failed to save."),
+                _("This could be because your disk is full, or you don't have permission to write to the configuration directory.  Any changes will be lost when you exit Xfmedia."),
+                GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
     }
     
     cleanup:
@@ -279,13 +271,13 @@ static void
 settings_ensure_branch(const gchar *branch)
 {
     gchar buf[2048], *p;
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_strlcpy(buf, branch, 2048);
     for(;;) {
         entry = g_hash_table_lookup(settings_hash, branch);
         if(!entry) {
-            entry = g_new(XfMediaConfigEntry, 1);
+            entry = g_new(XfmediaConfigEntry, 1);
             entry->key = g_strdup(branch);
             entry->type = XFMEDIA_SETTING_BRANCH;
             g_hash_table_insert(settings_hash, entry->key, entry);
@@ -302,10 +294,10 @@ settings_ensure_branch(const gchar *branch)
     }
 }
 
-static XfMediaConfigEntry *
+static XfmediaConfigEntry *
 settings_get_entry_for_set(const gchar *key)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     gchar *branch = g_strdup(key), *p;
     
     p = g_strrstr(branch, "/");
@@ -317,7 +309,7 @@ settings_get_entry_for_set(const gchar *key)
     
     entry = g_hash_table_lookup(settings_hash, key);
     if(!entry) {
-        entry = g_new0(XfMediaConfigEntry, 1);
+        entry = g_new0(XfmediaConfigEntry, 1);
         entry->key = g_strdup(key);
         settings_list = g_list_insert_sorted(settings_list, entry,
                 xfmedia_settings_glist_compare);
@@ -329,10 +321,10 @@ settings_get_entry_for_set(const gchar *key)
     return entry;
 }
 
-static XfMediaConfigEntry *
+static XfmediaConfigEntry *
 settings_get_entry_for_add(const gchar *key)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     gchar *branch, *p;
     
     entry = g_hash_table_lookup(settings_hash, key);
@@ -347,7 +339,7 @@ settings_get_entry_for_add(const gchar *key)
     }
     g_free(branch);
     
-    entry = g_new0(XfMediaConfigEntry, 1);
+    entry = g_new0(XfmediaConfigEntry, 1);
     entry->key = g_strdup(key);
     settings_list = g_list_insert_sorted(settings_list, entry,
             xfmedia_settings_glist_compare);
@@ -375,7 +367,7 @@ void
 xfmedia_settings_add_string(const gchar *key,
         const gchar *value)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_if_fail(key && settings_hash);
     
@@ -392,7 +384,7 @@ xfmedia_settings_add_string(const gchar *key,
 void
 xfmedia_settings_add_int(const gchar *key, gint value)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_if_fail(key && settings_hash);
     
@@ -409,7 +401,7 @@ xfmedia_settings_add_int(const gchar *key, gint value)
 void
 xfmedia_settings_add_bool(const gchar *key, gboolean value)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_if_fail(key && settings_hash);
     
@@ -426,7 +418,7 @@ xfmedia_settings_add_bool(const gchar *key, gboolean value)
 void
 xfmedia_settings_add_float(const gchar *key, gdouble value)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_if_fail(key && settings_hash);
     
@@ -443,7 +435,7 @@ xfmedia_settings_add_float(const gchar *key, gdouble value)
 G_CONST_RETURN gchar *
 xfmedia_settings_get_string(const gchar *key)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_val_if_fail(key && settings_hash, NULL);
     
@@ -457,7 +449,7 @@ xfmedia_settings_get_string(const gchar *key)
 gint
 xfmedia_settings_get_int(const gchar *key)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_val_if_fail(key && settings_hash, 0);    
     
@@ -471,7 +463,7 @@ xfmedia_settings_get_int(const gchar *key)
 gboolean
 xfmedia_settings_get_bool(const gchar *key)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_val_if_fail(key && settings_hash, FALSE);
     
@@ -485,7 +477,7 @@ xfmedia_settings_get_bool(const gchar *key)
 gdouble
 xfmedia_settings_get_float(const gchar *key)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_val_if_fail(key && settings_hash, 0.0);
     
@@ -499,7 +491,7 @@ xfmedia_settings_get_float(const gchar *key)
 void
 xfmedia_settings_set_string(const gchar *key, const gchar *value)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_if_fail(key && settings_hash);
     
@@ -512,7 +504,7 @@ xfmedia_settings_set_string(const gchar *key, const gchar *value)
 void
 xfmedia_settings_set_int(const gchar *key, gint value)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_if_fail(key && settings_hash);
     
@@ -525,7 +517,7 @@ xfmedia_settings_set_int(const gchar *key, gint value)
 void
 xfmedia_settings_set_bool(const gchar *key, gboolean value)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_if_fail(key && settings_hash);
     
@@ -538,7 +530,7 @@ xfmedia_settings_set_bool(const gchar *key, gboolean value)
 void
 xfmedia_settings_set_float(const gchar *key, gdouble value)
 {
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     g_return_if_fail(key && settings_hash);
     
@@ -585,6 +577,9 @@ xfmedia_settings_init()
     xfmedia_settings_add_bool("/xfmedia/general/dock_autohide", FALSE);
     xfmedia_settings_add_int("/xfmedia/general/vwin_pos_x", -1);
     xfmedia_settings_add_int("/xfmedia/general/vwin_pos_y", -1);
+    xfmedia_settings_add_bool("/xfmedia/general/show_tray_icon", FALSE);
+    xfmedia_settings_add_bool("/xfmedia/general/minimize_to_tray", TRUE);
+    xfmedia_settings_add_int("/xfmedia/general/tray_middle_click_action", 0);
     
 #if 0
     xfmedia_settings_add_int("/xfmedia/general/eq_band_0", 0);
@@ -611,11 +606,7 @@ xfmedia_settings_init()
     xfmedia_settings_add_bool("/xfmedia/playlist/save_on_exit_always", FALSE);
     xfmedia_settings_add_int("/xfmedia/playlist/metadata_load_impetus", 0);
     xfmedia_settings_add_bool("/xfmedia/playlist/repeat_one", FALSE);
-    
-    xfmedia_settings_add_int("/xfmedia/jtf/win_x", 25);
-    xfmedia_settings_add_int("/xfmedia/jtf/win_y", 25);
-    xfmedia_settings_add_int("/xfmedia/jtf/win_width", 425);
-    xfmedia_settings_add_int("/xfmedia/jtf/win_height", 200);
+    xfmedia_settings_add_int("/xfmedia/playlist/add_sort_type", 0);
     
     xfmedia_settings_add_int("/xfmedia/mediamarks/manager_x", 25);
     xfmedia_settings_add_int("/xfmedia/mediamarks/manager_y", 25);
@@ -635,13 +626,12 @@ get_default_playlist_file()
 }    
 
 gboolean
-xfmedia_settings_load_playlist(XfMediaMainwin *mwin)
+xfmedia_settings_load_playlist(XfmediaMainwin *mwin)
 {
     gchar *file = get_default_playlist_file();
     gboolean ret = FALSE;
     
     if(file) {
-        xfmedia_playlist_clear(mwin->plist);
         ret = xfmedia_playlists_load(mwin->plist, file);
         g_free(file);
     }
@@ -650,13 +640,13 @@ xfmedia_settings_load_playlist(XfMediaMainwin *mwin)
 }
 
 gboolean
-xfmedia_settings_save_playlist(XfMediaMainwin *mwin)
+xfmedia_settings_save_playlist(XfmediaMainwin *mwin)
 {
     gchar *file = get_default_playlist_file();
     gboolean ret = FALSE;
     
     if(file) {
-        ret = xfmedia_playlists_save(mwin->plist, file);
+        ret = xfmedia_playlists_save(mwin->plist, file, XFMEDIA_PLAYLIST_FORMAT_M3U);
         g_free(file);
     }
     
@@ -679,7 +669,7 @@ config_xml_start(GMarkupParseContext *context, const gchar *element_name,
     struct ConfigParserState *state = user_data;
     gint i;
     const gchar *name = NULL, *type = NULL, *value = NULL;
-    XfMediaConfigEntry *entry;
+    XfmediaConfigEntry *entry;
     
     for(i = 0; attribute_names[i]; i++) {
         if(!strcmp(attribute_names[i], "name"))
@@ -700,7 +690,7 @@ config_xml_start(GMarkupParseContext *context, const gchar *element_name,
             return;
         }
         
-        entry = g_new(XfMediaConfigEntry, 1);
+        entry = g_new(XfmediaConfigEntry, 1);
         entry->type = XFMEDIA_SETTING_BRANCH;
         entry->key = g_strconcat(state->cur_path,
                 (!strcmp(state->cur_path, "/")?"":"/"), name, NULL);
@@ -713,7 +703,7 @@ config_xml_start(GMarkupParseContext *context, const gchar *element_name,
     } else if(!strcmp(element_name, "setting")) {
         if(!name || !type || !value)
             return;
-        entry = g_new(XfMediaConfigEntry, 1);
+        entry = g_new(XfmediaConfigEntry, 1);
         entry->type = atoi(type);
         if(entry->type <= 0 || entry->type >= XFMEDIA_N_SETTING_TYPES) {
             g_free(entry);
