@@ -25,12 +25,12 @@
 
 G_BEGIN_DECLS
 
-typedef struct _XfMediaTrayIcon XfMediaTrayIcon;
-struct _XfMediaMainwin;
+typedef struct _XfmediaTrayIcon XfmediaTrayIcon;
+struct _XfmediaMainwin;
 
-XfMediaTrayIcon *xfmedia_tray_icon_create(GdkScreen *gscreen, struct _XfMediaMainwin *mwin, GtkMenu *menu);
-void xfmedia_tray_icon_set_tooltip(XfMediaTrayIcon *tray_icon, const gchar *tip);
-void xfmedia_tray_icon_destroy(XfMediaTrayIcon *tray_icon);
+XfmediaTrayIcon *xfmedia_tray_icon_create(GdkScreen *gscreen, struct _XfmediaMainwin *mwin, GtkMenu *menu);
+void xfmedia_tray_icon_set_tooltip(XfmediaTrayIcon *tray_icon, const gchar *tip);
+void xfmedia_tray_icon_destroy(XfmediaTrayIcon *tray_icon);
 
 G_END_DECLS
 
