@@ -36,21 +36,22 @@
 #include "xfmedia-internal.h"
 #include "main.h"
 #include "mainwin.h"
-#include "gtkxine.h"
+#include "xfmedia-xine.h"
 #include "jumptofilewin.h"
 #include "keybindings.h"
 #include <xfmedia/xfmedia-settings.h>
+#include <xfmedia/xfmedia-playlist-queue.h>
 
-typedef void (*XfMediaKeybindFunc)(XfMediaMainwin *mwin, gpointer user_data);
-#define KEYBIND_FUNC(name) static void name(XfMediaMainwin *mwin, gpointer user_data)
+typedef void (*XfmediaKeybindFunc)(XfmediaMainwin *mwin, gpointer user_data);
+#define KEYBIND_FUNC(name) static void name(XfmediaMainwin *mwin, gpointer user_data)
 
 typedef struct
 {
     gchar *displayname;
     gchar *keybind_id;
-    XfMediaKeybindFunc callback;
+    XfmediaKeybindFunc callback;
     gpointer user_data;
-} XfMediaKeybindEntry;
+} XfmediaKeybindEntry;
 
 KEYBIND_FUNC(kb_play)
 {
@@ -84,10 +85,13 @@ KEYBIND_FUNC(kb_quit)
 
 KEYBIND_FUNC(kb_fullscreen)
 {
-    gboolean is_fs = gtk_xine_is_fullscreen(mwin->gtx);
+    gboolean is_fs = xfmedia_xine_is_fullscreen(mwin->xfx);
     gint x, y;
-        
-    gtk_xine_set_fullscreen(mwin->gtx, !is_fs);
+    
+    if(!is_fs && !GTK_WIDGET_VISIBLE(mwin->video_window))
+        return;
+    
+    xfmedia_xine_set_fullscreen(mwin->xfx, !is_fs);
     if(!is_fs) {
         gtk_window_get_position(GTK_WINDOW(mwin->video_window), &x, &y);
         xfmedia_settings_set_int("/xfmedia/general/vwin_pos_x", x);
@@ -121,7 +125,7 @@ KEYBIND_FUNC(kb_audio_vis)
 
 KEYBIND_FUNC(kb_faster)
 {
-    gint cur_speed = gtk_xine_get_param(mwin->gtx, XINE_PARAM_SPEED);
+    gint cur_speed = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_SPEED);
     gint new_speed;
     
     switch(cur_speed) {
@@ -145,17 +149,17 @@ KEYBIND_FUNC(kb_faster)
     }
     
     if(cur_speed == XINE_SPEED_PAUSE) {
-        gtk_xine_set_param(mwin->gtx, XINE_PARAM_AUDIO_CLOSE_DEVICE, 0);
+        xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AUDIO_CLOSE_DEVICE, 0);
         xfmedia_mainwin_set_pause_buttons(mwin);
     }
-    gtk_xine_set_param(mwin->gtx, XINE_PARAM_SPEED, new_speed);
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_SPEED, new_speed);
     xfmedia_plugins_forward_signal_VOID("speed-changed");
 }
 
 
 KEYBIND_FUNC(kb_slower)
 {
-    gint cur_speed = gtk_xine_get_param(mwin->gtx, XINE_PARAM_SPEED);
+    gint cur_speed = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_SPEED);
     gint new_speed;
     
     switch(cur_speed) {
@@ -178,9 +182,9 @@ KEYBIND_FUNC(kb_slower)
             return;
     }
     
-    gtk_xine_set_param(mwin->gtx, XINE_PARAM_SPEED, new_speed);
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_SPEED, new_speed);
     if(new_speed == XINE_SPEED_PAUSE) {
-        gtk_xine_set_param(mwin->gtx, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
+        xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
         xfmedia_mainwin_set_play_buttons(mwin);
     }
     xfmedia_plugins_forward_signal_VOID("speed-changed");
@@ -190,14 +194,14 @@ KEYBIND_FUNC(kb_forward_10s)
 {
     gint pos_stream, pos_time, length_time, new_time;
     
-    if(gtk_xine_get_status(mwin->gtx) != XINE_STATUS_PLAY)
+    if(xfmedia_xine_get_status(mwin->xfx) != XINE_STATUS_PLAY)
         return;
     
-    if(gtk_xine_get_pos_length(mwin->gtx, &pos_stream, &pos_time, &length_time)) {
+    if(xfmedia_xine_get_pos_length(mwin->xfx, &pos_stream, &pos_time, &length_time)) {
         new_time = pos_time + 10000;
         if(new_time > length_time)
             new_time = length_time - 1000;
-        gtk_xine_play(mwin->gtx, 0, new_time);
+        xfmedia_xine_play(mwin->xfx, 0, new_time);
     }
 }
 
@@ -205,14 +209,14 @@ KEYBIND_FUNC(kb_forward_30s)
 {
     gint pos_stream, pos_time, length_time, new_time;
     
-    if(gtk_xine_get_status(mwin->gtx) != XINE_STATUS_PLAY)
+    if(xfmedia_xine_get_status(mwin->xfx) != XINE_STATUS_PLAY)
         return;
     
-    if(gtk_xine_get_pos_length(mwin->gtx, &pos_stream, &pos_time, &length_time)) {
+    if(xfmedia_xine_get_pos_length(mwin->xfx, &pos_stream, &pos_time, &length_time)) {
         new_time = pos_time + 30000;
         if(new_time > length_time)
             new_time = length_time - 1000;
-        gtk_xine_play(mwin->gtx, 0, new_time);
+        xfmedia_xine_play(mwin->xfx, 0, new_time);
     }
 }
 
@@ -220,14 +224,14 @@ KEYBIND_FUNC(kb_forward_60s)
 {
     gint pos_stream, pos_time, length_time, new_time;
     
-    if(gtk_xine_get_status(mwin->gtx) != XINE_STATUS_PLAY)
+    if(xfmedia_xine_get_status(mwin->xfx) != XINE_STATUS_PLAY)
         return;
     
-    if(gtk_xine_get_pos_length(mwin->gtx, &pos_stream, &pos_time, &length_time)) {
+    if(xfmedia_xine_get_pos_length(mwin->xfx, &pos_stream, &pos_time, &length_time)) {
         new_time = pos_time + 60000;
         if(new_time > length_time)
             new_time = length_time - 1000;
-        gtk_xine_play(mwin->gtx, 0, new_time);
+        xfmedia_xine_play(mwin->xfx, 0, new_time);
     }
 }
 
@@ -235,14 +239,14 @@ KEYBIND_FUNC(kb_backward_10s)
 {
     gint pos_stream, pos_time, length_time, new_time;
     
-    if(gtk_xine_get_status(mwin->gtx) != XINE_STATUS_PLAY)
+    if(xfmedia_xine_get_status(mwin->xfx) != XINE_STATUS_PLAY)
         return;
     
-    if(gtk_xine_get_pos_length(mwin->gtx, &pos_stream, &pos_time, &length_time)) {
+    if(xfmedia_xine_get_pos_length(mwin->xfx, &pos_stream, &pos_time, &length_time)) {
         new_time = pos_time - 10000;
         if(new_time < 0)
             new_time = 0;
-        gtk_xine_play(mwin->gtx, 0, new_time);
+        xfmedia_xine_play(mwin->xfx, 0, new_time);
     }
 }
 
@@ -250,14 +254,14 @@ KEYBIND_FUNC(kb_backward_30s)
 {
     gint pos_stream, pos_time, length_time, new_time;
     
-    if(gtk_xine_get_status(mwin->gtx) != XINE_STATUS_PLAY)
+    if(xfmedia_xine_get_status(mwin->xfx) != XINE_STATUS_PLAY)
         return;
     
-    if(gtk_xine_get_pos_length(mwin->gtx, &pos_stream, &pos_time, &length_time)) {
+    if(xfmedia_xine_get_pos_length(mwin->xfx, &pos_stream, &pos_time, &length_time)) {
         new_time = pos_time - 30000;
         if(new_time < 0)
             new_time = 0;
-        gtk_xine_play(mwin->gtx, 0, new_time);
+        xfmedia_xine_play(mwin->xfx, 0, new_time);
     }
 }
 
@@ -265,31 +269,43 @@ KEYBIND_FUNC(kb_backward_60s)
 {
     gint pos_stream, pos_time, length_time, new_time;
     
-    if(gtk_xine_get_status(mwin->gtx) != XINE_STATUS_PLAY)
+    if(xfmedia_xine_get_status(mwin->xfx) != XINE_STATUS_PLAY)
         return;
     
-    if(gtk_xine_get_pos_length(mwin->gtx, &pos_stream, &pos_time, &length_time)) {
+    if(xfmedia_xine_get_pos_length(mwin->xfx, &pos_stream, &pos_time, &length_time)) {
         new_time = pos_time - 60000;
         if(new_time < 0)
             new_time = 0;
-        gtk_xine_play(mwin->gtx, 0, new_time);
+        xfmedia_xine_play(mwin->xfx, 0, new_time);
     }
 }
 
+KEYBIND_FUNC(kb_enqueue)
+{
+    xfmedia_mainwin_queue_selected(mwin);
+}
+
+KEYBIND_FUNC(kb_show_queue)
+{
+    xfmedia_mainwin_toggle_playlist_queue(mwin);
+}
+
 KEYBIND_FUNC(kb_mute)
 {
-    gtk_xine_set_param(mwin->gtx, XINE_PARAM_AUDIO_MUTE,
-            !gtk_xine_get_param(mwin->gtx, XINE_PARAM_AUDIO_MUTE));
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AUDIO_MUTE,
+            !xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_AUDIO_MUTE));
 }
 
 KEYBIND_FUNC(kb_add_file)
 {
-    xfmedia_playlist_trigger_add_file(mwin->plist);
+    /* hack alert! */
+    xfmedia_mainwin_add_file_cb(NULL, mwin);
 }
 
 KEYBIND_FUNC(kb_add_url)
 {
-    xfmedia_playlist_trigger_add_url(mwin->plist);
+    /* hack alert! */
+    xfmedia_mainwin_add_url_cb(NULL, mwin);
 }
 
 KEYBIND_FUNC(kb_open_playlist)
@@ -311,26 +327,26 @@ KEYBIND_FUNC(kb_av_sync_plus)
 {
     gint cur_sync;
     
-    cur_sync = gtk_xine_get_param(mwin->gtx, XINE_PARAM_AV_OFFSET);
-    gtk_xine_set_param(mwin->gtx, XINE_PARAM_AV_OFFSET, cur_sync+3600);
+    cur_sync = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_AV_OFFSET);
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AV_OFFSET, cur_sync+3600);
 }
 
 KEYBIND_FUNC(kb_av_sync_minus)
 {
     gint cur_sync;
     
-    cur_sync = gtk_xine_get_param(mwin->gtx, XINE_PARAM_AV_OFFSET);
-    gtk_xine_set_param(mwin->gtx, XINE_PARAM_AV_OFFSET, cur_sync-3600);
+    cur_sync = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_AV_OFFSET);
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AV_OFFSET, cur_sync-3600);
 }
 
 KEYBIND_FUNC(kb_aspect)
 {
     gint aspect;
     
-    aspect = gtk_xine_get_param(mwin->gtx, XINE_PARAM_VO_ASPECT_RATIO);
+    aspect = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_VO_ASPECT_RATIO);
     if(++aspect >= XINE_VO_ASPECT_NUM_RATIOS)
         aspect = 0;
-    gtk_xine_set_param(mwin->gtx, XINE_PARAM_VO_ASPECT_RATIO, aspect);
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_VO_ASPECT_RATIO, aspect);
     DBG("set aspect code to %d", aspect);
 }
 
@@ -339,8 +355,91 @@ KEYBIND_FUNC(kb_jump_to_file)
     xfmedia_playlist_trigger_jump_to_file(mwin->plist);
 }
 
+KEYBIND_FUNC(kb_zoom_in)
+{
+    gint cur_zoom;
+    
+    if(!xfmedia_xine_is_fullscreen(mwin->xfx) || !GTK_WIDGET_VISIBLE(mwin->video_window))
+        return;
+    
+    cur_zoom = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_VO_ZOOM_X);
+    
+    cur_zoom += 20;
+    if(cur_zoom > XINE_VO_ZOOM_MAX)
+        return;
+    
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_VO_ZOOM_X, cur_zoom);
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_VO_ZOOM_Y, cur_zoom);
+}
 
-static XfMediaKeybindEntry keybind_entries[] =
+KEYBIND_FUNC(kb_zoom_out)
+{
+    gint cur_zoom;
+    
+    if(!xfmedia_xine_is_fullscreen(mwin->xfx) || !GTK_WIDGET_VISIBLE(mwin->video_window))
+        return;
+    
+    cur_zoom = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_VO_ZOOM_X);
+    
+    cur_zoom -= 20;
+    if(cur_zoom <= 0)
+        return;
+    
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_VO_ZOOM_X, cur_zoom);
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_VO_ZOOM_Y, cur_zoom);
+}
+
+KEYBIND_FUNC(kb_vwin_larger)
+{
+    gint w = -1, h = -1;
+    
+    if(xfmedia_xine_is_fullscreen(mwin->xfx) || !GTK_WIDGET_VISIBLE(mwin->video_window))
+        return;
+    
+    gtk_window_get_size(GTK_WINDOW(mwin->video_window), &w, &h);
+    if(w <= 0 || h <= 0)
+        return;
+    
+    gtk_window_resize(GTK_WINDOW(mwin->video_window), (gint)w*1.1, (gint)h*1.1);
+}
+
+KEYBIND_FUNC(kb_vwin_smaller)
+{
+    gint w = -1, h = -1;
+    
+    if(xfmedia_xine_is_fullscreen(mwin->xfx) || !GTK_WIDGET_VISIBLE(mwin->video_window))
+        return;
+    
+    gtk_window_get_size(GTK_WINDOW(mwin->video_window), &w, &h);
+    if(w <= 0 || h <= 0)
+        return;
+    
+    gtk_window_resize(GTK_WINDOW(mwin->video_window), (gint)w*0.9, (gint)h*0.9);
+}
+
+KEYBIND_FUNC(kb_vol_up)
+{
+    gint volume = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_AUDIO_VOLUME);
+    
+    volume += 2;
+    if(volume > 100)
+        volume = 100;
+    
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AUDIO_VOLUME, volume);
+}
+
+KEYBIND_FUNC(kb_vol_down)
+{
+    gint volume = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_AUDIO_VOLUME);
+    
+    volume -= 2;
+    if(volume < 0)
+        volume = 0;
+    
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AUDIO_VOLUME, volume);
+}
+
+static XfmediaKeybindEntry keybind_entries[] =
 {
     { N_("Play"), "play", kb_play, NULL },
     { N_("Pause"), "pause", kb_pause, NULL },
@@ -360,9 +459,8 @@ static XfMediaKeybindEntry keybind_entries[] =
     { N_("Backward 10 Seconds"), "backward-10s", kb_backward_10s, NULL },
     { N_("Backward 30 Seconds"), "backward-30s", kb_backward_30s, NULL },
     { N_("Backward 60 Seconds"), "backward-60s", kb_backward_60s, NULL },
-#if 0  /* unimplemented (yet) */
-    { N_("Enqueue Track"), "toggle-enqueue-item", kb_enqueue, NULL },
-#endif
+    { N_("Enqueue Track"), "enqueue-track", kb_enqueue, NULL },
+    { N_("Show Queued Tracks"), "show-queue", kb_show_queue, NULL },
     { N_("Toggle Audio Mute"), "toggle-mute", kb_mute, NULL },
     { N_("Add File to Playlist"), "add-file", kb_add_file, NULL },
     { N_("Add URL to Playlist"), "add-url", kb_add_url, NULL },
@@ -372,11 +470,17 @@ static XfMediaKeybindEntry keybind_entries[] =
     { N_("A/V Sync Plus"), "av-sync-plus", kb_av_sync_plus, NULL },
     { N_("A/V Sync Minus"), "av-sync-minus", kb_av_sync_minus, NULL },
     { N_("Select Aspect Ratio"), "toggle-aspect", kb_aspect, NULL },
-    { N_("Jump to File"), "jump-to-file", kb_jump_to_file, NULL }
+    { N_("Jump to File"), "jump-to-file", kb_jump_to_file, NULL },
+    { N_("Video Zoom In"), "vid-zoom-in", kb_zoom_in, NULL },
+    { N_("Video Zoom Out"), "vid-zoom-out", kb_zoom_out, NULL },
+    { N_("Video Window Larger"), "vid-win-larger", kb_vwin_larger, NULL },
+    { N_("Video Window Smaller"), "vid-win-smaller", kb_vwin_smaller, NULL },
+    { N_("Volume Up"), "vol-up", kb_vol_up, NULL },
+    { N_("Volume Down"), "vol-down", kb_vol_down, NULL }
 };
 
 void
-xfmedia_keybindings_activate(XfMediaMainwin *mwin, const gchar *keybind_id)
+xfmedia_keybindings_activate(XfmediaMainwin *mwin, const gchar *keybind_id)
 {
     /* FIXME: put all this stuff in a GHashTable? */
     gint i;
@@ -450,7 +554,7 @@ keybind_keypress_cb(GtkWidget *w, GdkEventKey *evt, gpointer user_data)
 }
 
 static void
-keybind_show_set_shortcut_dialog(XfMediaMainwin *mwin, const gchar *displayname,
+keybind_show_set_shortcut_dialog(XfmediaMainwin *mwin, const gchar *displayname,
         const gchar *shortcut, GtkListStore *ls, GtkTreeIter *itr,
         GtkWindow *parent)
 {
@@ -502,14 +606,26 @@ keybind_show_set_shortcut_dialog(XfMediaMainwin *mwin, const gchar *displayname,
                 xfmedia_keybindings_modify(shortcut, kinfo.new_keyval,
                         kinfo.new_modmask);
                 gtk_list_store_set(ls, itr, KB_LIST_KEYBIND, str, -1);
-                xfmedia_keybindings_save(NULL);
+                if(!xfmedia_keybindings_save(NULL)) {
+                    xfce_message_dialog(parent, _("Keybindings Save Error"),
+                            GTK_STOCK_DIALOG_ERROR,
+                            _("The keybindings file failed to save."),
+                            _("This could be because your disk is full, or you don't have permission to write to the configuration directory.  Any changes will be lost when you quit Xfmedia."),
+                            GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
+                }
                 g_free(str);
             }
             break;
         case GTK_RESPONSE_NO:
             xfmedia_keybindings_modify(shortcut, 0, 0);
             gtk_list_store_set(ls, itr, KB_LIST_KEYBIND, _("None"), -1);
-            xfmedia_keybindings_save(NULL);
+            if(!xfmedia_keybindings_save(NULL)) {
+                xfce_message_dialog(parent, _("Keybindings Save Error"),
+                        GTK_STOCK_DIALOG_ERROR,
+                        _("The keybindings file failed to save."),
+                        _("This could be because your disk is full, or you don't have permission to write to the configuration directory.  Any changes will be lost when you quit Xfmedia."),
+                        GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
+            }
             break;
         default:
             break;
@@ -523,7 +639,7 @@ keybind_show_set_shortcut_dialog(XfMediaMainwin *mwin, const gchar *displayname,
 gboolean
 keybind_btnpress_cb(GtkWidget *w, GdkEventButton *evt, gpointer user_data)
 {
-    XfMediaMainwin *mwin = user_data;
+    XfmediaMainwin *mwin = user_data;
     
     if(evt->button == 1 && evt->type == GDK_2BUTTON_PRESS) {
         GtkTreePath *path = NULL;
@@ -556,7 +672,7 @@ keybind_btnpress_cb(GtkWidget *w, GdkEventButton *evt, gpointer user_data)
 
 /* FIXME: this is slow.  hashtable? */
 static G_CONST_RETURN gchar *
-kb_lookup_name(XfMediaKeybinding *kb)
+kb_lookup_name(XfmediaKeybinding *kb)
 {
     gint i;
     
@@ -569,7 +685,7 @@ kb_lookup_name(XfMediaKeybinding *kb)
 }
 
 GtkWidget *
-xfmedia_keybindings_create_editor(XfMediaMainwin *mwin, GtkWidget *parent)
+xfmedia_keybindings_create_editor(XfmediaMainwin *mwin, GtkWidget *parent)
 {
     GtkWidget *topvbox, *sw, *treeview;
     GtkListStore *ls;
@@ -592,7 +708,7 @@ xfmedia_keybindings_create_editor(XfMediaMainwin *mwin, GtkWidget *parent)
             G_TYPE_STRING);
     keybindings = xfmedia_keybindings_get_list();
     for(l = keybindings; l; l = l->next) {
-        XfMediaKeybinding *kb = l->data;
+        XfmediaKeybinding *kb = l->data;
         gchar *kb_str;
         const gchar *kb_displayname;
         
