@@ -26,8 +26,8 @@
 
 G_BEGIN_DECLS
 
-void xfmedia_keybindings_activate(XfMediaMainwin *mwin, const gchar *keybind_id);
-GtkWidget *xfmedia_keybindings_create_editor(XfMediaMainwin *mwin, GtkWidget *parent);
+void xfmedia_keybindings_activate(XfmediaMainwin *mwin, const gchar *keybind_id);
+GtkWidget *xfmedia_keybindings_create_editor(XfmediaMainwin *mwin, GtkWidget *parent);
 
 G_END_DECLS
 
